@@ -1,5 +1,5 @@
 #/bin/sh
-package=$(sed -n 's/^project\s*(\s*\(.*\)\s*).*/\1/p' CMakeLists.txt)
+package=$(sed -n 's/^project\s*(\s*\(\w*\).*/\1/p' CMakeLists.txt)
 version=$(sed -n 's/.*VERSION "\(.*\)".*/\1/p' CMakeLists.txt)
 
 pkg="$package-$version"
@@ -18,7 +18,7 @@ svn export . $pkg || die "export failed"
 cd "$pkg" || die "could not cd into package"
 
 # ensure pot is up-to-date
-( cd po && ./update-po.sh ) || die "could not update po files"
+( cd po && VERSION_CONTROL=none ./update-po.sh ) || die "could not update po files"
 
 ( mkdir build &&
   cd build &&
