@@ -1,6 +1,6 @@
 # This file is part of the Frescobaldi project, http://www.frescobaldi.org/
 #
-# Copyright (c) 2008, 2009  Wilbert Berendsen
+# Copyright (c) 2008, 2009, 2010 by Wilbert Berendsen
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,16 +17,18 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
-import os, re, sip, time
+import os, re, sip, weakref
 from dbus.service import method
 
-from PyQt4.QtCore import QObject, QString, QTimer, QVariant, Qt, SIGNAL
+from PyQt4.QtCore import (
+    QEvent, QObject, QSize, QString, QTimer, QVariant, Qt, SIGNAL)
 from PyQt4.QtGui import (
-    QActionGroup, QLabel, QProgressBar, QStackedWidget, QWidget)
+    QActionGroup, QColor, QIcon, QLabel, QPalette, QPixmap, QProgressBar,
+    QStackedWidget, QWidget)
 from PyKDE4.kdecore import KConfig, KGlobal, KUrl, i18n
 from PyKDE4.kdeui import (
-    KActionMenu, KApplication, KDialog, KIcon, KLineEdit, KMenu, KMessageBox,
-    KStandardAction, KVBox)
+    KActionMenu, KApplication, KDialog, KIcon, KIconLoader, KLineEdit, KMenu,
+    KMessageBox, KStandardAction, KVBox)
 from PyKDE4.kparts import KParts
 from PyKDE4.ktexteditor import KTextEditor
 
@@ -93,12 +95,8 @@ class MainApp(kateshell.app.MainApp):
         return Document(self, url, encoding)
 
     def defaultDirectory(self):
-        return config().readPathEntry("default directory", "")
+        return unicode(config().readPathEntry("default directory", ""))
     
-    @lazymethod
-    def stateManager(self):
-        return StateManager(self)
-
     def findDocument(self, url):
         """
         Finds the document at KUrl url.
@@ -110,10 +108,12 @@ class MainApp(kateshell.app.MainApp):
         # now check for our LocalFileManagers ...
         for d in self.documents:
             manager = d.localFileManager()
-            if manager:
-                if manager.path() == url:
-                    return d
+            if manager and manager.path() == url:
+                return d
         return False
+    
+    def keepMetaInfo(self):
+        return config().readEntry("save metainfo", QVariant(False)).toBool()
 
 
 class Document(kateshell.app.Document):
@@ -136,9 +136,6 @@ class Document(kateshell.app.Document):
             action = self.view.actionCollection().action(name)
             if action:
                 sip.delete(action)
-        # read state information (view settings etc.)
-        if config().readEntry("save metainfo", QVariant(False)).toBool():
-            self.app.stateManager().loadState(self)
         # activate completion
         iface = self.view.codeCompletionInterface()
         if iface:
@@ -146,7 +143,7 @@ class Document(kateshell.app.Document):
 
     @lazymethod
     def completionModel(self):
-        return CompletionModel(self.view)
+        return CompletionModel(self)
         
     def contextMenu(self):
         menu = KMenu(self.view)
@@ -157,20 +154,21 @@ class Document(kateshell.app.Document):
         self.manipulator().populateContextMenu(self.view.contextMenu())
     
     def aboutToClose(self):
-        if config().readEntry("save metainfo", QVariant(False)).toBool():
-            self.app.stateManager().saveState(self)
         self.resetLocalFileManager()
-        
+    
     def setCursorPosition(self, line, column, translate=True):
         shiftPressed = KApplication.keyboardModifiers() & Qt.ShiftModifier
-        if self.view and shiftPressed:
-            # select from the current cursor position.
-            start = self.view.cursorPosition()
+        if self.view:
+            if shiftPressed:
+                # select from the current cursor position.
+                start = self.view.cursorPosition()
+            else:
+                self.view.removeSelection()
         super(Document, self).setCursorPosition(line, column, translate)
         if self.view and shiftPressed:
             end = self.view.cursorPosition()
             self.view.setSelection(KTextEditor.Range(start, end))
-            self.manipulator().fixSelection()
+            self.manipulator().adjustSelectionToChords()
     
     def variables(self):
         """
@@ -202,6 +200,21 @@ class Document(kateshell.app.Document):
         import frescobaldi_app.document
         return frescobaldi_app.document.DocumentManipulator(self)
 
+    def currentIndent(self, cursor=None, checkColumn=True):
+        """
+        Returns the indent of the line the given cursor is on, or
+        of the current line.
+        
+        If checkColumn is True (default), the indent returned is not deeper
+        than the column the cursor is on.
+        """
+        if cursor is None:
+            cursor = self.view.cursorPosition()
+        text = self.line(cursor.line())
+        if checkColumn:
+            text = text[:cursor.column()]
+        return len(re.match(r'\s*', text).group().expandtabs(self.tabWidth()))
+        
     def indent(self, text, start = None, startscheme = False):
         """
         Convenience method to indent text according to settings of this
@@ -229,25 +242,104 @@ class Document(kateshell.app.Document):
         self._localFileManager = None
 
 
-class MainWindow(kateshell.mainwindow.MainWindow):
+class SymbolManager(object):
+    """
+    Manages the LilyPond icons for objects like widgets and actions.
+    
+    Ensures that if the palette is changed, the icons are redrawn in the current
+    foreground color. You can mixin this with a widget or just instantiate it
+    and call the recolor method on palette changes (look at our changeEvent
+    method for an example).
+    """
+    def __init__(self):
+        # we store mapping obj->(icon_name, size) in a weak dict
+        # objs that have a setIcon method
+        self._objs = {}
+        # objs that have a setItemIcon method: obj->(index, icon_name, size)
+        self._itemobjs = {}
+        # objs that need to have setIconSize called after drawing the icons
+        self._sizeobjs = {}
+        self._ownsize = None # must setIconSize() be called on us?
+        # TODO: add support for models or objects with a special property
+        self._defaultSize = 22
+        self._timer = QTimer()
+        self._timer.setSingleShot(True)
+        QObject.connect(self._timer, SIGNAL("timeout()"), self.redrawSymbols)
+        self.triggerRedraw()
+
+    def addSymbol(self, obj, icon_name, size=0):
+        self._objs[obj] = (icon_name, size)
+    
+    def addItemSymbol(self, obj, index, icon_name, size=0):
+        self._itemobjs.setdefault(obj, []).append((index, icon_name, size))
+        
+    def setSymbolSize(self, obj, size):
+        if obj is self:
+            self._ownsize = size
+        else:
+            self._sizeobjs[obj] = size
+        
+    def defaultSymbolSize(self):
+        return self._defaultSize
+    
+    def setDefaultSymbolSize(self, size):
+        self._defaultSize = size
+        
+    def triggerRedraw(self, msecs = 0):
+        self._timer.start(msecs)
+        
+    def redrawSymbols(self):
+        for obj, (icon_name, size) in self._objs.items()[:]: # copy
+            try:
+                obj.setIcon(self.symbolIcon(icon_name, size))
+            except RuntimeError: # underlying C/C++ object has been deleted
+                del self._objs[obj]
+        for obj, items in self._itemobjs.items()[:]: # copy
+            try:
+                for index, icon_name, size in items:
+                    obj.setItemIcon(index, self.symbolIcon(icon_name, size))
+            except RuntimeError:  # underlying C/C++ object has been deleted
+                del self._itemobjs[obj]
+        for obj, size in self._sizeobjs.items()[:]: # copy
+            try:
+                obj.setIconSize(QSize(size, size))
+            except RuntimeError:  # underlying C/C++ object has been deleted
+                del self._sizeobjs[obj]
+        if self._ownsize:
+            try:
+                self.setIconSize(QSize(self._ownsize, self._ownsize))
+            except RuntimeError:  # underlying C/C++ object has been deleted
+                self._ownsize = None
+            
+    def symbolIcon(self, name, size=0):
+        size = size or self._defaultSize
+        pixmap = KIconLoader.global_().loadIcon(name, KIconLoader.User, size) 
+        alpha = pixmap.alphaChannel()
+        pixmap.fill(KApplication.palette().color(QPalette.ButtonText))
+        pixmap.setAlphaChannel(alpha)
+        return QIcon(pixmap)
+        
+    def changeEvent(self, ev):
+        """
+        Respond to events, in particular palette events,
+        to recolor LilyPond symbol icons.
+        """
+        if ev.type() == QEvent.PaletteChange:
+            self.triggerRedraw()
+
+
+class MainWindow(SymbolManager, kateshell.mainwindow.MainWindow):
     """ Our customized Frescobaldi MainWindow """
     def __init__(self, app):
+        SymbolManager.__init__(self)
         kateshell.mainwindow.MainWindow.__init__(self, app)
-        
         self.progressBar = QProgressBar()
         self.progressBar.setMaximumHeight(16)
         self.statusBar().addPermanentWidget(self.progressBar)
         self.progressBar.hide()
-
-        KonsoleTool(self)
-        LogTool(self)
-        QuickInsertTool(self)
-        RumorTool(self)
-        if not config().readEntry("disable pdf preview", QVariant(False)).toBool():
-            PDFTool(self)
-        
         self.currentDocumentChanged.connect(self.updateJobActions)
-        
+        self.expansionShortcuts = ExpansionShortcuts(self)
+    
     @lazymethod
     def actionManager(self):
         """
@@ -287,6 +379,11 @@ class MainWindow(kateshell.mainwindow.MainWindow):
         import frescobaldi_app.progress
         return frescobaldi_app.progress.ProgressBarManager(jobmanager,
             self.progressBar)
+    
+    @lazymethod
+    def blankStaffPaperWizard(self):
+        import frescobaldi_app.blankpaper
+        return frescobaldi_app.blankpaper.Dialog(self)
         
     def setupActions(self):
         super(MainWindow, self).setupActions()
@@ -340,6 +437,8 @@ class MainWindow(kateshell.mainwindow.MainWindow):
                 log = self.tools["log"].log(d)
                 if log:
                     self.actionManager().addActionsToLog(result, log)
+                    if not success:
+                        log.show() # even if LP didn't show an error location
             log = self.tools["log"].createLog(d)
             job = self.jobManager().createJob(d, log, preview)
             job.done.connect(finished)
@@ -363,11 +462,16 @@ class MainWindow(kateshell.mainwindow.MainWindow):
         def edit_repeat_last():
             self.currentDocument().manipulator().repeatLastExpression()
             
-        @self.onAction(i18n("Expand"), key="Ctrl+.",
+        @self.onAction(i18n("Insert or Manage Expansions..."), key="Ctrl+.",
             tooltip=i18n("Expand last word or open the expansions dialog."))
         def edit_expand():
             self.expandManager().expand()
         
+        # (this action is currently only displayed in the contextmenu)
+        @self.onSelAction(i18n("Add to Expansions"), "list-add")
+        def edit_expand_add(text):
+            self.expandManager().addExpansion(text)
+        
         @self.onAction(i18n("Next blank line"), "go-down-search", key="Alt+Down",
             tooltip=i18n("Go to the next blank line."))
         def edit_next_blank_line():
@@ -376,6 +480,7 @@ class MainWindow(kateshell.mainwindow.MainWindow):
                 if not isblank(d.line(num)) and isblank(d.line(num + 1)):
                     d.view.setCursorPosition(
                         KTextEditor.Cursor(num + 1, len(d.line(num + 1))))
+                    d.view.removeSelection()
                     return
         
         @self.onAction(i18n("Previous blank line"), "go-up-search", key="Alt+Up",
@@ -385,17 +490,97 @@ class MainWindow(kateshell.mainwindow.MainWindow):
             for num in range(d.view.cursorPosition().line() - 1, -1, -1):
                 if isblank(d.line(num)) and (num == 0 or not isblank(d.line(num - 1))):
                     d.view.setCursorPosition(KTextEditor.Cursor(num, len(d.line(num))))
+                    d.view.removeSelection()
                     return
         
+        @self.onAction(i18n("Select to next blank line"), key="Shift+Alt+Down",
+            tooltip=i18n(
+            "Selects text from the current position down to and including the next blank line."))
+        def edit_select_next_blank_line():
+            d = self.currentDocument()
+            cursor = d.view.cursorPosition()
+            for num in range(cursor.line() + 1, d.doc.lines() - 2):
+                if isblank(d.line(num)) and not isblank(d.line(num + 1)):
+                    newcur = KTextEditor.Cursor(num + 1, 0)
+                    break
+            else:
+                docRange = d.doc.documentRange()
+                newcur = docRange.end()
+            if d.view.selection():
+                r = d.view.selectionRange()
+                if cursor.position() == r.start().position():
+                    cursor = r.end()
+                elif cursor.position() == r.end().position():
+                    cursor = r.start()
+            else:
+                line = cursor.line()
+                while line < d.doc.lines() and isblank(d.line(line)):
+                    line += 1
+                cursor.setLine(line)
+            d.view.setSelection(KTextEditor.Range(cursor, newcur))
+            d.view.setCursorPosition(newcur)
+        
+        @self.onAction(i18n("Select to previous blank line"), key="Shift+Alt+Up",
+            tooltip=i18n(
+            "Selects text from the current position up to right after the previous blank line."))
+        def edit_select_previous_blank_line():
+            d = self.currentDocument()
+            cursor = d.view.cursorPosition()
+            for num in range(cursor.line() - 2, -1, -1):
+                if isblank(d.line(num)) and not isblank(d.line(num + 1)):
+                    newcur = KTextEditor.Cursor(num + 1, 0)
+                    break
+            else:
+                newcur = KTextEditor.Cursor(0, 0)
+            if d.view.selection():
+                r = d.view.selectionRange()
+                if cursor.position() == r.start().position():
+                    cursor = r.end()
+                elif cursor.position() == r.end().position():
+                    cursor = r.start()
+            else:
+                line = cursor.line()
+                if line < d.doc.lines() - 1 and isblank(d.line(line)):
+                    line += 1
+                cursor.setLine(line)
+            d.view.setSelection(KTextEditor.Range(cursor, newcur))
+            d.view.setCursorPosition(newcur)
+        
+        @self.onSelAction(i18n("Move selection to next blank line"), key="Shift+Alt+Ctrl+Down",
+            tooltip=i18n("Moves selected block to next blank line."))
+        def edit_moveto_next_blank_line(text):
+            self.currentDocument().manipulator().moveSelectionDown()
+        
+        @self.onSelAction(i18n("Move selection to previous blank line"), key="Shift+Alt+Ctrl+Up",
+            tooltip=i18n("Moves selected block to previous blank line."))
+        def edit_moveto_previous_blank_line(text):
+            self.currentDocument().manipulator().moveSelectionUp()
+        
         # actions and functionality for editing pitches
         a = KActionMenu(KIcon("applications-education-language"),
                 i18n("Pitch Name Language"), self)
         a.setToolTip(i18n("Change the LilyPond language used for pitch names "
                           "in this document or in the selection."))
         self.actionCollection().addAction('pitch_change_language', a)
-        QObject.connect(a.menu(), SIGNAL("aboutToShow()"), lambda menu=a.menu():
+        self.pitchNameLanguageMenu = menu = a.menu() # Save because of BUG SIP 4.10 loosing outer scope
+        QObject.connect(menu, SIGNAL("aboutToShow()"), lambda:
             self.currentDocument().manipulator().populateLanguageMenu(menu))
         
+        @self.onAction(i18n("Convert Relative to &Absolute"), tooltip=i18n(
+            "Converts the notes in the document or selection from relative to absolute pitch."))
+        def pitch_relative_to_absolute():
+            self.currentDocument().manipulator().convertRelativeToAbsolute()
+            
+        @self.onAction(i18n("Convert Absolute to &Relative"), tooltip=i18n(
+            "Converts the notes in the document or selection from absolute to relative pitch."))
+        def pitch_absolute_to_relative():
+            self.currentDocument().manipulator().convertAbsoluteToRelative()
+
+        @self.onAction(i18n("Transpose..."), tooltip=i18n(
+            "Transposes all notes in the document or selection."))
+        def pitch_transpose():
+            self.currentDocument().manipulator().transpose()
+        
         # actions and functionality for editing rhythms
         @self.onSelAction(i18n("Double durations"),
             tooltip=i18n("Double all the durations in the selection."))
@@ -465,6 +650,28 @@ class MainWindow(kateshell.mainwindow.MainWindow):
             rhythm = unicode(KApplication.clipboard().text())
             return ly.duration.applyRhythm(text, rhythm)
         
+        # Bar lines
+        for name, text, title, key in (
+            ("bar_double", "||", i18n("Double bar line"), "Alt+;"),
+            ("bar_end", "|.", i18n("Ending bar line"), "Alt+."),
+            ("bar_dotted", ":", i18n("Dotted bar line"), None),
+            ("bar_dashed", "dashed", i18n("Dashed bar line"), None),
+            ("bar_invisible", "", i18n("Invisible bar line"), None),
+            ("bar_repeat_start", "|:", i18n("Repeat start"), None),
+            ("bar_repeat_double", ":|:", i18n("Repeat both"), None),
+            ("bar_repeat_end", ":|", i18n("Repeat end"), None),
+            ("bar_tick", "'", i18n("Tick bar line"), "Alt+'"),
+            ("bar_single", "|", i18n("Single bar line"), None),
+            ("bar_sws", "|.|", i18n("Small-Wide-Small bar line"), None),
+            ("bar_ws", ".|", i18n("Wide-Small bar line"), None),
+            ("bar_ww", ".|.", i18n("Double wide bar line"), None),
+            ("bar_cswc", ":|.:", i18n("Repeat both (old)"), None),
+            ("bar_cswsc", ":|.|:", i18n("Repeat both (classic)"), None),
+            ):
+            a = self.act(name, title, key=key, func=lambda text=text:
+                    self.currentDocument().manipulator().insertBarLine(text))
+            self.addSymbol(a, name)
+            
         # Setup lyrics hyphen and de-hyphen action
         @self.onSelAction(i18n("Hyphenate Lyrics Text"), keepSelection=False, key="Ctrl+L")
         def lyrics_hyphen(text):
@@ -511,12 +718,42 @@ class MainWindow(kateshell.mainwindow.MainWindow):
         def options_configure():
             import frescobaldi_app.settings
             frescobaldi_app.settings.SettingsDialog(self).show()
+        
+        @self.onSelAction(i18n("Repeat selected music"), key="Ctrl+Shift+R",
+            keepSelection=False)
+        def edit_repeat(text):
+            return self.currentDocument().manipulator().wrapSelection(text, 
+                "\\repeat volta 2 {")
+        
+        @self.onAction(i18n("Insert pair of braces"), "code-context", key="Ctrl+{")
+        def edit_insert_braces():
+            d = self.currentDocument()
+            if d.selectionText():
+                d.replaceSelectionWith(d.manipulator().wrapSelection(
+                    d.selectionText()), keepSelection=False)
+            else:
+                self.currentDocument().manipulator().insertTemplate("{\n(|)\n}")
+        
+        # Other wizards/tools:
+        @self.onAction(i18n("Create blank staff paper"), "text-plain")
+        def wizard_blank_staff_paper():
+            self.blankStaffPaperWizard().show()
 
+            
+    def setupTools(self):
+        KonsoleTool(self)
+        LogTool(self)
+        QuickInsertTool(self)
+        RumorTool(self)
+        if not config().readEntry("disable pdf preview", QVariant(False)).toBool():
+            PDFTool(self)
+        LilyDocTool(self)
+    
     def setupGeneratedMenus(self):
         super(MainWindow, self).setupGeneratedMenus()
-        # Generated file menu:
-        menu = self.factory().container("lilypond_actions", self)
-        def populateGenFilesMenu(menu=menu):
+        # Generated file menu, save because of SIP 4.10 BUG, loosing outer scope
+        self.generatedLilyPondFilesMenu = menu = self.factory().container("lilypond_actions", self)
+        def populateGenFilesMenu():
             for action in menu.actions():
                 if action.objectName() != "actions_email":
                     sip.delete(action)
@@ -543,6 +780,11 @@ class MainWindow(kateshell.mainwindow.MainWindow):
         act("lilypond_runner").setIcon(KIcon(icon))
         act("lilypond_runner").setToolTip(tip)
 
+    def addActionCollectionsToShortcutsDialog(self, dlg):
+        super(MainWindow, self).addActionCollectionsToShortcutsDialog(dlg)
+        dlg.addCollection(self.expansionShortcuts.actionCollection(),
+            i18n("Expansion Manager"))
+    
     def saveSettings(self):
         self.app.stateManager().cleanup()
         super(MainWindow, self).saveSettings()
@@ -580,6 +822,37 @@ class ApplyRhythmDialog(KDialog):
         self.lineedit.selectAll()
 
 
+# Work around a SIP 4.9.1 bug causing a crash when logging out of the built-in
+# terminal tool and then hiding and showing the tool. The destroyed() signal of
+# the part is not caught, so that SIP does not notice the part has been deleted,
+# let alone Frescobaldi.
+#
+# Now we catch the disappearing of the part by embedding the part in another
+# widget and catching the ChildRemoved event.
+class KonsoleContainer(QWidget):
+    def __init__(self, tool, partWidget):
+        self.tool = tool
+        QWidget.__init__(self)
+        from PyQt4.QtGui import QVBoxLayout
+        l = QVBoxLayout(self)
+        l.addWidget(partWidget)
+        l.setContentsMargins(0, 0, 0, 0)
+        self.setFocusProxy(partWidget)
+        
+    def childEvent(self, ev):
+        if ev.type() == QEvent.ChildRemoved:
+            QTimer.singleShot(0, self.childRemoved)
+
+    def childRemoved(self):
+        self.tool.slotDestroyed()
+        try:
+            sip.delete(self)
+        except RuntimeError:
+            # underlying C/C++ object has been deleted, can occur
+            # if exiting Frescobaldi with terminal floating
+            pass
+
+
 class KonsoleTool(kateshell.mainwindow.KPartTool):
     """ A tool embedding a Konsole """
     _partlibrary = "libkonsolepart"
@@ -587,30 +860,36 @@ class KonsoleTool(kateshell.mainwindow.KPartTool):
     def __init__(self, mainwin):
         kateshell.mainwindow.KPartTool.__init__(self, mainwin,
             "konsole", i18n("Terminal"), "terminal",
-            dock=kateshell.mainwindow.Bottom)
+            key="Meta+Alt+T", dock=kateshell.mainwindow.Bottom)
         mainwin.currentDocumentChanged.connect(self.sync)
-            
+    
+    # Workaround for the SIP 4.9.1 destroyed-not-caught bug:
     def factory(self):
-        w = super(KonsoleTool, self).factory()
-        if self.part:
-            d = self.mainwin.currentDocument()
-            if d and not d.url().isEmpty():
-                url = d.url()
-            else:
-                url = KUrl.fromPath(
-                    self.mainwin.app.defaultDirectory() or os.getcwd())
-            self.openUrl(url)
-        return w
+        widget = super(KonsoleTool, self).factory()
+        if widget and self.part:
+            QObject.disconnect(self.part, SIGNAL("destroyed()"), self.slotDestroyed)
+            return KonsoleContainer(self, widget)
+        
+    def partLoaded(self):
+        d = self.mainwin.currentDocument()
+        if d and not d.url().isEmpty():
+            url = d.url()
+        else:
+            url = KUrl.fromPath(
+                self.mainwin.app.defaultDirectory() or os.getcwd())
+        self.openUrl(url)
 
+    def slotAction(self):
+        if self.part and self.isActive() and not self.widget.hasFocus():
+            self.widget.setFocus()
+        else:
+            self.toggle()
+    
     def show(self):
         super(KonsoleTool, self).show()
         if self.part:
-            self.part.widget().setFocus()
+            self.widget.setFocus()
         
-    def hide(self):
-        super(KonsoleTool, self).hide()
-        self.mainwin.view().setFocus()
-
     def sync(self, doc):
         if (self.part and self._sync
             and doc and doc.doc and not doc.url().isEmpty()):
@@ -618,7 +897,6 @@ class KonsoleTool(kateshell.mainwindow.KPartTool):
             self.openUrl(doc.url())
 
     def addMenuActions(self, m):
-        m.addSeparator()
         a = m.addAction(i18n("S&ynchronize Terminal with Current Document"))
         a.setCheckable(True)
         a.setChecked(self._sync)
@@ -641,7 +919,7 @@ class PDFTool(kateshell.mainwindow.KPartTool):
         self._config = {}
         kateshell.mainwindow.KPartTool.__init__(self, mainwin,
             "pdf", i18n("PDF Preview"), "application-pdf",
-            dock=kateshell.mainwindow.Right)
+            key="Meta+Alt+P", dock=kateshell.mainwindow.Right)
         mainwin.currentDocumentChanged.connect(self.sync)
         self._currentUrl = None
         # We open urls with a timer otherwise Okular is called 
@@ -650,6 +928,7 @@ class PDFTool(kateshell.mainwindow.KPartTool):
         self._timer.setSingleShot(True)
         self._timer.setInterval(200)
         QObject.connect(self._timer, SIGNAL("timeout()"), self.timeoutFunc)
+        mainwin.aboutToClose.connect(self._timer.stop)
     
     def timeoutFunc(self):
         if self._currentUrl:
@@ -662,7 +941,6 @@ class PDFTool(kateshell.mainwindow.KPartTool):
                 self.openUrl(KUrl(pdfs[0]))
 
     def addMenuActions(self, m):
-        m.addSeparator()
         def act(name, title):
             a = m.addAction(title)
             a.setCheckable(True)
@@ -675,7 +953,14 @@ class PDFTool(kateshell.mainwindow.KPartTool):
         QObject.connect(a, SIGNAL("triggered()"), self.openOkularSettings)
         m.addSeparator()
         act("sync", i18n("S&ynchronize Preview with Current Document"))
+        a = m.addAction(i18n("Reload"))
+        QObject.connect(a, SIGNAL("triggered()"), self.reloadUrl)
+        a.setEnabled(bool(self._currentUrl))
 
+    def reloadUrl(self):
+        if self._currentUrl:
+            self._timer.start()
+            
     def openOkularSettings(self):
         self.materialize()
         if self.part:
@@ -711,7 +996,7 @@ class PDFTool(kateshell.mainwindow.KPartTool):
         self.part.actionCollection().action("show_leftpanel").setChecked(
             self._config["leftpanel"])
         # change shortcut context for actions that conflict with Kate's
-        for action in "view_scroll_up", "view_scroll_down":
+        for action in "view_scroll_up", "view_scroll_down", "close_find_bar":
             self.part.actionCollection().action(action).setShortcutContext(
                 Qt.WidgetShortcut)
         # default to single page layout
@@ -737,7 +1022,7 @@ class QuickInsertTool(kateshell.mainwindow.Tool):
     def __init__(self, mainwin):
         kateshell.mainwindow.Tool.__init__(self, mainwin,
             "quickinsert", i18n("Quick Insert"), "document-properties",
-            dock=kateshell.mainwindow.Left)
+            key="Meta+Alt+I", dock=kateshell.mainwindow.Left)
             
     def factory(self):
         import frescobaldi_app.lqi
@@ -749,7 +1034,7 @@ class LogTool(kateshell.mainwindow.Tool):
         self._config = {}
         kateshell.mainwindow.Tool.__init__(self, mainwin,
             "log", i18n("LilyPond Log"), "run-lilypond",
-            dock=kateshell.mainwindow.Bottom,
+            key="Meta+Alt+L", dock=kateshell.mainwindow.Bottom,
             widget=QStackedWidget())
         self.logs = {}
         self.widget.addWidget(QLabel("<center>(%s)</center>" % i18n("no log")))
@@ -785,7 +1070,6 @@ class LogTool(kateshell.mainwindow.Tool):
                 self.hide()
     
     def addMenuActions(self, m):
-        m.addSeparator()
         def act(name, title):
             a = m.addAction(title)
             a.setCheckable(True)
@@ -793,6 +1077,10 @@ class LogTool(kateshell.mainwindow.Tool):
             QObject.connect(a, SIGNAL("triggered()"),
                 lambda: self.toggleAction(name))
         act("errors only", i18n("Only show on errors"))
+        # context menu options of the Log, if there...
+        # (check it without importing runlily module)
+        if hasattr(self.widget.currentWidget(), "addContextMenuActions"):
+            self.widget.currentWidget().addContextMenuActions(m)
         
     def toggleAction(self, name):
         self._config[name] = not self._config[name]
@@ -813,95 +1101,84 @@ class RumorTool(kateshell.mainwindow.Tool):
     def __init__(self, mainwin):
         kateshell.mainwindow.Tool.__init__(self, mainwin,
             "rumor", i18n("Rumor"), "media-record",
-            dock=kateshell.mainwindow.Bottom)
+            key="Meta+Alt+R", dock=kateshell.mainwindow.Bottom)
             
     def factory(self):
         import frescobaldi_app.rumor
         return frescobaldi_app.rumor.RumorPanel(self)
 
 
-class StateManager(object):
-    """
-    Manages state and meta-info for documents, like bookmarks
-    and cursor position, etc.
-    """
-    def __init__(self, app):
-        self.app = app
-        self.metainfos = KConfig("metainfos", KConfig.NoGlobals, "appdata")
+class LilyDocTool(kateshell.mainwindow.Tool):
+    helpAnchor = "lilydoc"
+    def __init__(self, mainwin):
+        kateshell.mainwindow.Tool.__init__(self, mainwin,
+            "lilydoc", i18n("LilyPond Documentation"), "lilydoc",
+            key="Meta+Alt+D", dock=kateshell.mainwindow.Right)
+        self._docFinder = None
         
-    def loadState(self, doc):
-        if (not doc.url().isEmpty() and
-                self.metainfos.hasGroup(doc.url().prettyUrl())):
-            group = self.metainfos.group(doc.url().prettyUrl())
-            last = group.readEntry("time", QVariant(0.0)).toDouble()[0]
-            # when it is a local file, only load the state when the
-            # file was not modified later
-            if not doc.localPath() or (
-                    os.path.exists(doc.localPath()) and
-                    os.path.getmtime(doc.localPath()) <= last):
-                doc.readConfig(group)
-            
-    def saveState(self, doc):
-        if doc.view and not doc.url().isEmpty():
-            group = self.metainfos.group(doc.url().prettyUrl())
-            group.writeEntry("time", QVariant(time.time()))
-            doc.writeConfig(group)
-            group.sync()
-            
-    def cleanup(self):
-        """ Purge entries that are not used for more than a month. """
-        for g in self.metainfos.groupList():
-            last = self.metainfos.group(g).readEntry("time", QVariant(0.0)).toDouble()[0]
-            if (time.time() - last) / 86400 > 31:
-                self.metainfos.deleteGroup(g)
-        self.metainfos.sync()
+    def factory(self):
+        self.docFinder()
+        import frescobaldi_app.lilydoc
+        return frescobaldi_app.lilydoc.LilyDoc(self)
+
+    def docFinder(self):
+        if self._docFinder is None:
+            self.newDocFinder()
+        return self._docFinder
+        
+    def newDocFinder(self):
+        import frescobaldi_app.lilydoc
+        self._docFinder = frescobaldi_app.lilydoc.DocFinder(self)
+        
+    def openUrl(self, url):
+        self.materialize()
+        self.widget.openUrl(url)
+        self.show()
 
 
 class CompletionModel(KTextEditor.CodeCompletionModel):
-    def __init__(self, parent):
-        KTextEditor.CodeCompletionModel.__init__(self, parent)
-        self.matches = []
-
-    def index(self, row, column, parent):
-        if (row < 0 or row >= len(self.matches) or
-            column < 0 or column >= KTextEditor.CodeCompletionModel.ColumnCount or
-            parent.isValid()):
-            return QModelIndex()
-        return self.createIndex(row, column, 0)
+    def __init__(self, doc):
+        KTextEditor.CodeCompletionModel.__init__(self, doc.view)
+        self.doc = weakref.proxy(doc)
+        self.result = None
         
-    def data(self, index, role):
-        if index.column() != KTextEditor.CodeCompletionModel.Name:
-            return QVariant()
-        if role == Qt.DisplayRole:
-            return QVariant(self.matches[index.row()])
-        elif role == KTextEditor.CodeCompletionModel.CompletionRole:
-            return QVariant(
-                KTextEditor.CodeCompletionModel.FirstProperty |
-                KTextEditor.CodeCompletionModel.Public |
-                KTextEditor.CodeCompletionModel.LastProperty )
-        elif role == KTextEditor.CodeCompletionModel.ScopeIndex:
-            return QVariant(0)
-        elif role == KTextEditor.CodeCompletionModel.MatchQuality:
-            return QVariant(10)
-        elif role == KTextEditor.CodeCompletionModel.HighlightingMethod:
-            return QVariant(QVariant.Invalid)
-        elif role == KTextEditor.CodeCompletionModel.InheritanceDepth:
-            return QVariant(0)
-        else:
-            return QVariant()
-            
-    def rowCount(self, parent):
-        if parent.isValid():
-            return 0 # Do not make the model look hierarchical
-        else:
-            return len(self.matches)
-            
     def completionInvoked(self, view, word, invocationType):
         import frescobaldi_app.completion
-        self.matches = frescobaldi_app.completion.findMatches(
-            view, word, invocationType) or []
+        self.result = frescobaldi_app.completion.getCompletions(
+            self, view, word, invocationType)
+        self.reset()
+    
+    def index(self, row, column, parent):
+        return self.result.index(row, column, parent)
+        
+    def data(self, index, role):
+        return self.result.data(index, role)
+    
+    def executeCompletionItem(self, doc, word, row):
+        if not self.result.executeCompletionItem(doc, word, row):
+            KTextEditor.CodeCompletionModel.executeCompletionItem(
+                self, doc, word, row)
+    
+    def rowCount(self, parent):
+        return self.result.rowCount(parent)
 
 
+class ExpansionShortcuts(kateshell.app.UserShortcuts):
+    """
+    Manages shortcuts for the expand dialog.
+    This is setup initially so that keyboard shortcuts for expansions
+    work without the expandManager being loaded. Pressing a keyboard
+    shortcut actually loads the expansion manager to perform the action.
+    """
+    # which config group to store our shortcuts
+    configGroup = "expand shortcuts"
+    
+    def widget(self):
+        return self.mainwin.viewStack # where the text editor views are.
+        
+    def target(self):
+        return self.mainwin.expandManager()
+
 
 # Easily get our global config
 def config(group="preferences"):
