@@ -1,6 +1,6 @@
 # This file is part of the Frescobaldi project, http://www.frescobaldi.org/
 #
-# Copyright (c) 2008, 2009  Wilbert Berendsen
+# Copyright (c) 2008, 2009, 2010 by Wilbert Berendsen
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -156,11 +156,11 @@ class RumorPanel(QWidget):
     def saveSettings(self):
         conf = config("rumor")
         conf.writeEntry("tempo", QVariant(self.tempo.tempo()))
-        conf.writeEntry("quantize", self.quantize.currentText())
+        conf.writeEntry("quantize", QVariant(unicode(self.quantize.currentText())))
         conf.writeEntry("step", QVariant(self.step.isChecked()))
         conf.writeEntry("mono", QVariant(self.mono.isChecked()))
-        conf.writeEntry("meter", autofy(self.meter.currentText()))
-        conf.writeEntry("keysig", autofy(self.keysig.currentText()))
+        conf.writeEntry("meter", QVariant(unicode(autofy(self.meter.currentText()))))
+        conf.writeEntry("keysig", QVariant(unicode(autofy(self.keysig.currentText()))))
         conf.writeEntry("timidity", QVariant(self.timidity.isChecked()))
         self.showMessage(i18n("Settings have been saved."), 1000)
 
@@ -242,7 +242,7 @@ class RumorPanel(QWidget):
             else:
                 acc = 0
         else:
-            acc == int(acc)
+            acc = int(acc)
         acc += 2    # use sharps for half tones leading to second, fifth, sixth
         args.append("--key=%s" % ly.key.num2key[lang][bound(acc, -8, 12)])
 
@@ -411,6 +411,7 @@ class RumorSettings(KDialog):
     """
     def __init__(self, mainwin):
         KDialog.__init__(self, mainwin)
+        self.setAttribute(Qt.WA_DeleteOnClose)
         self.setCaption(i18n("Rumor Settings"))
         self.setButtons(KDialog.ButtonCode(
             KDialog.Ok | KDialog.Cancel | KDialog.Help))
@@ -529,7 +530,7 @@ class RumorSettings(KDialog):
             idefault = odefault = 'oss:1'
         else:
             idefault = 'kbd'
-            odefault = self.olist[max(1, len(self.olist)-1)]
+            odefault = self.olist and self.olist[-1] or ""
         i = conf.readEntry("midi in", QVariant(idefault)).toString()
         o = conf.readEntry("midi out", QVariant(odefault)).toString()
         if i in self.ilist:
@@ -564,9 +565,9 @@ class RumorSettings(KDialog):
     def saveSettings(self):
         """ Save the settings """
         conf = config("rumor")
-        conf.writeEntry("midi in", self.ilist[self.ibut.currentIndex()])
-        conf.writeEntry("midi out", self.olist[self.obut.currentIndex()])
-        conf.writeEntry("language", autofy(self.lang.currentText()))
+        conf.writeEntry("midi in", QVariant(self.ilist[self.ibut.currentIndex()]))
+        conf.writeEntry("midi out", QVariant(self.olist[self.obut.currentIndex()]))
+        conf.writeEntry("language", QVariant(unicode(autofy(self.lang.currentText()))))
         conf.writeEntry("absolute pitches", QVariant(self.absPitches.isChecked()))
         conf.writeEntry("explicit durations", QVariant(self.explDur.isChecked()))
         conf.writeEntry("no barlines", QVariant(self.noBar.isChecked()))
@@ -578,8 +579,8 @@ class RumorSettings(KDialog):
         for row in range(self.scripts.topLevelItemCount()):
             item = self.scripts.topLevelItem(row)
             if item.checkState(0) == Qt.Checked:
-                names.append(item.text(0))
-        conf.writeEntry("scripts", names)
+                names.append(unicode(item.text(0)))
+        conf.writeEntry("scripts", QVariant(names))
 
 
 
@@ -592,14 +593,20 @@ def parseAconnect(channel):
     option = channel == 'i' and '--input' or '--output'
     cmd = unicode(config("commands").readEntry("aconnect", QVariant("aconnect")).toString())
     res = []
-    for line in Popen([cmd, option], stdout=PIPE).communicate()[0].splitlines():
+    # run aconnect in the english (standard language)
+    env = dict(os.environ)
+    for key in "LANG", "LC_ALL", "LC_MESSAGES":
+        if key in env:
+            del env[key]
+    for line in Popen([cmd, option], stdout=PIPE, env=env).communicate()[0].splitlines():
         m = re.match(r"client\s*(\d+)|\s*(\d+)\s+'([^']+)'", line)
-        if m.group(1):
-            client = m.group(1)
-        elif client != "0":
-            port = "%s:%s" % (client, m.group(2))
-            name = "%s (%s)" % (port, m.group(3).strip())
-            res.append((port, name))
+        if m:
+            if m.group(1):
+                client = m.group(1)
+            elif client != "0":
+                port = "%s:%s" % (client, m.group(2))
+                name = "%s (%s)" % (port, m.group(3).strip())
+                res.append((port, name))
     return res
 
 def getOSSnrMIDIs():
