@@ -1,6 +1,6 @@
 # This file is part of the Frescobaldi project, http://www.frescobaldi.org/
 #
-# Copyright (c) 2008, 2009  Wilbert Berendsen
+# Copyright (c) 2008, 2009, 2010 by Wilbert Berendsen
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -24,13 +24,17 @@ import math, os, re, shutil, sys, tempfile, time
 from PyQt4.QtCore import (
     QObject, QProcess, QSize, QTimer, QUrl, QVariant, Qt, SIGNAL)
 from PyQt4.QtGui import (
-    QBrush, QColor, QFont, QFrame, QTextBrowser, QTextCharFormat, QTextCursor,
-    QToolBar, QVBoxLayout)
-from PyKDE4.kdecore import KGlobal, KProcess, i18n
+    QBrush, QColor, QFont, QFrame, QStackedWidget, QTextBrowser,
+    QTextCharFormat, QTextCursor, QToolBar, QVBoxLayout, QWidget)
+from PyKDE4.kdecore import KGlobal, KPluginLoader, KProcess, KUrl, i18n
+from PyKDE4.kdeui import (
+    KApplication, KDialog, KGuiItem, KIcon, KMenu, KMessageBox, KStandardGuiItem)
+from PyKDE4.kio import KEncodingFileDialog
 
 from signals import Signal
 
 from kateshell.app import resolvetabs_text
+from frescobaldi_app.actions import openPDF
 import frescobaldi_app.mainapp
 
 def config(group):
@@ -63,7 +67,11 @@ class Ly2PDF(object):
         self.p = KProcess()
         self.p.setOutputChannelMode(KProcess.MergedChannels)
         self.p.setWorkingDirectory(self.directory)
-        cmd = [config("commands").readEntry("lilypond", QVariant("lilypond")).toString(), "--pdf"]
+        cmd = [config("commands").readEntry("lilypond", QVariant("lilypond")).toString()]
+        if config("preferences").readEntry("verbose lilypond output",
+                                           QVariant(False)).toBool():
+            cmd.append("--verbose")
+        cmd.append("--pdf")
         cmd.append(self.preview and "-dpoint-and-click" or "-dno-point-and-click")
         if config("preferences").readEntry("delete intermediate files",
                                            QVariant(True)).toBool():
@@ -79,7 +87,7 @@ class Ly2PDF(object):
         self.log.clear()
         mode = unicode(self.preview and i18n("preview mode") or i18n("publish mode"))
         self.log.writeLine(i18n("LilyPond [%1] starting (%2)...", self.lyfile_arg, mode))
-        self.startTime = math.floor(time.time())
+        self.startTime = time.time()
         self.p.start()
         
     def finished(self, exitCode, exitStatus):
@@ -90,8 +98,14 @@ class Ly2PDF(object):
             self.log.writeMsg(i18n("LilyPond [%1] exited with exit status %2.",
                 self.lyfile_arg, exitStatus), "msgerr")
         else:
-            self.log.writeMsg(i18n("LilyPond [%1] finished.", self.lyfile_arg),
-                "msgok")
+            # We finished successfully, show elapsed time...
+            seconds = time.time() - self.startTime
+            if seconds < 60:
+                elapsed = '%.1f"' % seconds
+            else:
+                elapsed = "%i'%i\"" % divmod(seconds, 60)
+            self.log.writeMsg(i18n("LilyPond [%1] finished (%2).",
+                self.lyfile_arg, elapsed), "msgok")
         self.bye(not (exitCode or exitStatus))
     
     def error(self, errCode):
@@ -132,13 +146,11 @@ class Ly2PDF(object):
         
     def readOutput(self):
         encoding = sys.getfilesystemencoding() or 'utf-8'
-        text = str(self.p.readAllStandardOutput()).decode(encoding)
+        text = str(self.p.readAllStandardOutput()).decode(encoding, 'replace')
         parts = _ly_message_re.split(text)
         # parts has an odd length(1, 6, 11 etc)
         # message, <url, path, line, col, message> etc.
         self.log.write(parts.pop(0))
-        if parts:
-            self.log.show() # warnings or errors will be printed
         while len(parts[:5]) == 5:
             url, path, line, col, msg = parts[:5]
             path = os.path.join(self.directory, path)
@@ -152,7 +164,8 @@ class Ly2PDF(object):
         """
         Returns a function that can list updated files based on extension.
         """
-        return frescobaldi_app.mainapp.updatedFiles(self.lyfile, self.startTime)
+        return frescobaldi_app.mainapp.updatedFiles(self.lyfile,
+            math.floor(self.startTime))
         
 
 class LyDoc2PDF(Ly2PDF):
@@ -286,6 +299,11 @@ class Log(LogWidget):
         LogWidget.__init__(self, tool.widget)
         QObject.connect(self.textBrowser, SIGNAL("anchorClicked(QUrl)"),
             self.anchorClicked)
+        # context menu:
+        self.textBrowser.setContextMenuPolicy(Qt.CustomContextMenu)
+        QObject.connect(self.textBrowser,
+            SIGNAL("customContextMenuRequested(QPoint)"),
+            self.showContextMenu)
     
     def clear(self):
         self.anchors.clear()
@@ -304,12 +322,75 @@ class Log(LogWidget):
         f.setAnchorHref(anchor)
         f.setToolTip(tooltip or i18n("Click to edit this file"))
         self.write(text, format)
+        self.show() # because this refers to a warning or error
     
     def anchorClicked(self, url):
         ref = self.anchors.get(str(url.path()))
         if ref:
             ref.activate()
     
+    def showContextMenu(self, pos):
+        m = KMenu(self.textBrowser)
+        m.addTitle(i18n("LilyPond Log"))
+        self.addContextMenuActions(m)
+        m.exec_(self.textBrowser.mapToGlobal(pos))
+        
+    def addContextMenuActions(self, menu):
+        a = menu.addAction(KIcon("edit-copy"), i18n("&Copy"))
+        QObject.connect(a, SIGNAL("triggered()"), self.copyLog)
+        g = KStandardGuiItem.saveAs()
+        a = menu.addAction(g.icon(), g.text())
+        QObject.connect(a, SIGNAL("triggered()"), self.saveLogAs)
+
+    def copyLog(self):
+        text = (self.textBrowser.textCursor().selection().toPlainText()
+                or self.textBrowser.toPlainText())
+        if text:
+            KApplication.clipboard().setText(text)
+        
+    def saveLogAs(self):
+        startDir, fileName = os.path.split(self.doc.localPath())
+        fileName = (os.path.splitext(fileName)[0] or "lilypond") + ".log"
+        dlg = KEncodingFileDialog(startDir, 'utf-8', '',
+            i18n("Save LilyPond Log as"),
+            KEncodingFileDialog.Saving, self.textBrowser)
+        dlg.setSelection(fileName)
+        
+        # because KEncodingFileDialog.setConfirmOverwrite() is not available
+        # in KDE 4.1, we try to use it, but fall back to our old own
+        # implementation.
+        checkOverwrite = False
+        try:
+            dlg.setConfirmOverwrite(True)
+        except AttributeError:
+            checkOverwrite = True
+        result = dlg.exec_()
+        if not result:
+            return # Cancelled
+        
+        # now confirm overwrite if the dialog didn't handle that (KDE < 4.2)
+        fileName = unicode(dlg.selectedFile())
+        if (checkOverwrite and os.path.exists(fileName)
+            and KMessageBox.warningContinueCancel(self.textBrowser,
+                i18n("A file named \"%1\" already exists. "
+                        "Are you sure you want to overwrite it?",
+                        os.path.basename(fileName)),
+                i18n("Overwrite File?"), KGuiItem(i18n("&Overwrite"))) ==
+                KMessageBox.Cancel):
+            return
+        
+        encoding = str(dlg.selectedEncoding())
+        text = unicode(self.textBrowser.textCursor().selection().toPlainText()
+                or self.textBrowser.toPlainText())
+        try:
+            f = open(fileName, 'w')
+            f.write(text.encode(encoding, 'replace'))
+            f.write('\n')
+            f.close()
+        except (OSError, IOError), e:
+            KMessageBox.error(self.textBrowser,
+                i18n("Could not save LilyPond log:\n\n%1", unicode(e)))
+
 
 class FileRef(object):
     """
@@ -422,6 +503,153 @@ class LocalFileManager(object):
         return lyfile
  
 
+class BackgroundJob(object):
+    """
+    Manages LilyPond jobs in the background. Can display a dialog with the log
+    output if there was an error.
+    Subclass this at your liking.
+    """
+    def __init__(self, log=None):
+        self.log = log or LogWidget()
+        self._directory = None
+        self.job = None
+        
+    def directory(self):
+        if self._directory is None:
+            self._directory = tempfile.mkdtemp()
+        return self._directory
+        
+    def run(self, text, fileName='output.ly'):
+        lyfile = os.path.join(self.directory(), fileName)
+        file(lyfile, 'w').write(text.encode('utf-8'))
+        # ... and run LilyPond.
+        self.job = Ly2PDF(lyfile, self.log)
+        self.job.done.connect(self.finished)
+    
+    def finished(self):
+        """
+        Called when the job is done.
+        """
+        pass
+    
+    def cleanup(self):
+        """
+        Stop a job if running and remove temporary files.
+        """
+        if self.job:
+            self.job.done.disconnect(self.finished)
+            self.job.abort()
+            self.job = None
+        if self._directory:
+            shutil.rmtree(self._directory)
+            self._directory = None
+    
+    def showLog(self, message, title='', parent=None):
+        """
+        Show the log in a simple modal dialog.
+        """
+        dlg = KDialog(parent)
+        if title:
+            dlg.setCaption(title)
+        dlg.setButtons(KDialog.ButtonCode(KDialog.Close))
+        dlg.setMainWidget(self.log)
+        self.log.writeMsg(message, 'msgerr')
+        dlg.setInitialSize(QSize(500, 300))
+        return dlg.exec_()
+            
+
+class LilyPreviewWidget(BackgroundJob, QStackedWidget):
+    """
+    A widget that can display a string of LilyPond code as a PDF.
+    If the code is changed, the PDF is automagically rebuilt.
+    Also the signal done(success) is then emitted.
+    """
+    def __init__(self, *args):
+        QStackedWidget.__init__(self, *args)
+        BackgroundJob.__init__(self)
+        # The widget stack has two widgets, a log and a PDF preview.
+        # the Log is already created in BackgroundJob
+        self.addWidget(self.log)
+        self.setCurrentWidget(self.log)
+        
+        # the PDF preview, load Okular part.
+        # If not, we just run the default PDF viewer.
+        self.part = None
+        factory = KPluginLoader("okularpart").factory()
+        if factory:
+            part = factory.create(self)
+            if part:
+                self.part = part
+                self.addWidget(part.widget())
+                self.setCurrentWidget(part.widget())
+                # hide mini pager
+                w = part.widget().findChild(QWidget, "miniBar")
+                if w:
+                    w.parent().hide()
+                # hide left panel
+                a = part.actionCollection().action("show_leftpanel")
+                if a and a.isChecked():
+                    a.toggle()
+                # default to single page layout
+                a = part.actionCollection().action("view_render_mode_single")
+                if a and not a.isChecked():
+                    a.trigger()
+                # change shortcut context for this one (bound to Esc)
+                a = part.actionCollection().action("close_find_bar")
+                if a:
+                    a.setShortcutContext(Qt.WidgetShortcut)
+            
+    def preview(self, text):
+        """
+        Runs LilyPond on the text and update the preview.
+        """
+        if self.job:
+            self.job.disconnect(self.finished)
+            self.job.abort()
+        self.run(text, 'preview.ly')
+        self.setCurrentWidget(self.log)
+    
+    def finished(self):
+        pdfs = self.job.updatedFiles()("pdf")
+        if pdfs:
+            self.openPDF(pdfs[0])
+        self.job = None
+
+    def openPDF(self, fileName):
+        if self.part:
+            if self.part.openUrl(KUrl.fromPath(fileName)):
+                self.setCurrentWidget(self.part.widget())
+        else:
+            openPDF(fileName, self.window())
+
+    
+class LilyPreviewDialog(KDialog):
+    def __init__(self, parent):
+        KDialog.__init__(self, parent)
+        self.setCaption(i18n("PDF Preview"))
+        self.setButtons(KDialog.ButtonCode(KDialog.Close))
+        self.preview = LilyPreviewWidget(self)
+        self.setMainWidget(self.preview)
+        self.setMinimumSize(QSize(400, 300))
+        self.loadSettings()
+        QObject.connect(self, SIGNAL("finished()"), self.slotFinished)
+        
+    def loadSettings(self):
+        self.restoreDialogSize(config("preview dialog"))
+        
+    def saveSettings(self):
+        self.saveDialogSize(config("preview dialog"))
+        
+    def slotFinished(self):
+        self.saveSettings()
+        self.preview.cleanup()
+    
+    def showPreview(self, ly):
+        self.preview.preview(ly)
+        self.exec_()
+
+
+
 def textFormats():
     """ Return a dict with text formats for the log view """
     log = QTextCharFormat()
