@@ -1,6 +1,6 @@
 # This file is part of the Frescobaldi project, http://www.frescobaldi.org/
 #
-# Copyright (c) 2008, 2009  Wilbert Berendsen
+# Copyright (c) 2008, 2009, 2010 by Wilbert Berendsen
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -21,7 +21,7 @@
 Score Wizard
 """
 
-import os, re, sip, string, sys
+import os, re, sip, string, sys, types
 import ly, ly.dom
 from rational import Rational
 
@@ -37,7 +37,8 @@ from PyKDE4.kdeui import (
     KStandardGuiItem, KVBox)
 
 from kateshell.app import lazymethod
-from frescobaldi_app.widgets import TapButton, LilyIcon
+from frescobaldi_app.mainapp import SymbolManager
+from frescobaldi_app.widgets import TapButton
 from frescobaldi_app.version import defaultVersion
 
 def config(group=None):
@@ -233,6 +234,7 @@ class Parts(QSplitter):
         allParts.setRootIsDecorated(False)
         allParts.headerItem().setHidden(True)
         score.setSelectionMode(QListWidget.ExtendedSelection)
+        score.setDragDropMode(QListWidget.InternalMove)
 
         class PartItem(QListWidgetItem):
             """
@@ -320,7 +322,7 @@ class Parts(QSplitter):
         for name, parts in categories():
             group = QTreeWidgetItem(allParts, [name])
             group.setFlags(Qt.ItemIsEnabled)
-            #group.setExpanded(True)
+            group.setIcon(0, KIcon("inode-directory"))
             for part in parts:
                 p = QTreeWidgetItem(group, [part.name()])
                 p.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable)
@@ -336,12 +338,13 @@ class Parts(QSplitter):
         return [self.score.item(row).part for row in range(self.score.count())]
 
 
-class Settings(QWidget):
+class Settings(SymbolManager, QWidget):
     """
     The widget where users can set other preferences.
     """
     def __init__(self, parent):
         QWidget.__init__(self, parent)
+        SymbolManager.__init__(self)
         parent.addPage(self, i18n("Score settings"))
 
         h = QHBoxLayout(self)
@@ -374,13 +377,15 @@ class Settings(QWidget):
         l = QLabel(i18n("Time signature:"), h)
         self.time = QComboBox(h)
         self.time.setEditable(True)
-        self.time.addItem(LilyIcon('time_c44'), '(4/4)')
-        self.time.addItem(LilyIcon('time_c22'), '(2/2)')
         self.time.addItems([
+            '(4/4)', '(2/2)',
             '2/4', '3/4', '4/4', '5/4', '6/4', '7/4',
             '2/2', '3/2', '4/2',
             '3/8', '5/8', '6/8', '7/8', '8/8', '9/8', '12/8',
             '3/16', '6/16', '12/16'])
+        # palette sensitive icons for the first two items
+        self.addItemSymbol(self.time, 0, 'time_c44')
+        self.addItemSymbol(self.time, 1, 'time_c22')
         l.setBuddy(self.time)
 
         h = KHBox()
@@ -389,9 +394,10 @@ class Settings(QWidget):
         self.pickup = QComboBox(h)
         self.pickup.addItem(i18n("None"))
         self.pickup.insertSeparator(1)
-        durs = [(LilyIcon('note_%s' % d.replace('.', 'd')), d) for d in durations]
+        durs = [('note_%s' % d.replace('.', 'd'), d) for d in durations]
         for icon, text in durs:
-            self.pickup.addItem(icon, text)
+            self.addItemSymbol(self.pickup, self.pickup.count(), icon)
+            self.pickup.addItem(text)
         l.setBuddy(self.pickup)
 
         h = KHBox()
@@ -401,7 +407,8 @@ class Settings(QWidget):
 
         l.setBuddy(self.metroDur)
         for icon, text in durs:
-            self.metroDur.addItem(icon, '')
+            self.addItemSymbol(self.metroDur, self.metroDur.count(), icon)
+            self.metroDur.addItem('')
         l = QLabel('=', h)
         l.setAlignment(Qt.AlignCenter)
         l.setMaximumWidth(20)
@@ -504,7 +511,7 @@ class Settings(QWidget):
         l.setBuddy(self.paper)
         self.paperLandscape = QCheckBox(i18n("Landscape"), h)
         self.paper.addItem(i18n("Default"))
-        self.paper.addItems(paperSizes)
+        self.paper.addItems(ly.paperSizes)
         @onSignal(self.paper, "activated(int)")
         def checkLandscape(i):
             self.paperLandscape.setEnabled(bool(i))
@@ -557,7 +564,7 @@ class Settings(QWidget):
         conf.writeEntry('midi', QVariant(self.midi.isChecked()))
         conf.writeEntry('metronome mark', QVariant(self.metro.isChecked()))
         if self.paper.currentIndex() > 0:
-            conf.writeEntry('paper size', paperSizes[self.paper.currentIndex() - 1])
+            conf.writeEntry('paper size', ly.paperSizes[self.paper.currentIndex() - 1])
         conf.writeEntry('paper landscape', QVariant(self.paperLandscape.isChecked()))
         g = config('instrument names')
         g.writeEntry('show', QVariant(self.instr.isChecked()))
@@ -575,10 +582,10 @@ class Settings(QWidget):
         self.metro.setChecked(conf.readEntry('metronome mark', QVariant(False)).toBool())
 
         psize = conf.readEntry('paper size', QVariant('')).toString()
-        if psize in paperSizes:
-            self.paper.setCurrentIndex(paperSizes.index(psize) + 1)
+        if psize in ly.paperSizes:
+            self.paper.setCurrentIndex(ly.paperSizes.index(psize) + 1)
         self.paperLandscape.setChecked(conf.readEntry('paper landscape', QVariant(False)).toBool())
-        self.paperLandscape.setEnabled(psize in paperSizes)
+        self.paperLandscape.setEnabled(psize in ly.paperSizes)
 
         g = config('instrument names')
         def readconf(entry, itemlist, defaultIndex):
@@ -646,6 +653,8 @@ class Builder(object):
     Parts may interact with:
     
     include             to request filenames to be included
+    
+    addCodeBlock        to add arbitrary strings to the output (e.g. functions)
 
     lilypondVersion     a tuple like (2, 11, 64) describing the LilyPond the
                         document is built for.
@@ -653,7 +662,9 @@ class Builder(object):
     getInstrumentNames  to translate instrument names
 
     setInstrumentNames  to translate and set instrument names for a node
-
+    
+    midi                property is True if MIDI output is requested
+    
     setMidiInstrument   to set the Midi instrument for a node
     """
     def __init__(self, wizard):
@@ -697,6 +708,9 @@ class Builder(object):
         # keep track of include files:
         self.includeFiles = []
         
+        # keep track of arbitrary code blocks:
+        self.codeBlocks = []
+        
         # version:
         version = unicode(s.lyversion.currentText())
         ly.dom.Version(version, doc)
@@ -723,6 +737,9 @@ class Builder(object):
                 ly.dom.Paper(doc)).after = 1
             ly.dom.BlankLine(doc)
 
+        # insert code blocks here later
+        codeBlockOffset = len(doc)
+
         # get the part list
         parts = self.wizard.parts.partList()
         if parts:
@@ -733,6 +750,12 @@ class Builder(object):
         if language:
             self.include("%s.ly" % language)
 
+        # add code blocks, if any:
+        for code in self.codeBlocks[::-1]:
+            node = isinstance(code, basestring) and ly.dom.Line(code) or code()
+            node.after = 2
+            doc.insert(codeBlockOffset, node)
+        
         # add the files that want to be included at the beginning
         if self.includeFiles:
             doc.insert(2, ly.dom.BlankLine())
@@ -890,6 +913,26 @@ class Builder(object):
         # We don't use a set, because we want to maintain the order.
         if fileName not in self.includeFiles:
             self.includeFiles.append(fileName)
+    
+    def addCodeBlock(self, code):
+        """
+        Adds an arbitary Node to the output file, containing e.g. Scheme
+        functions.
+        
+        The argument is either a string, a function or an iterable (containing
+        other strings, functions and/or iterables).
+        
+        Strings and functions are recursively added to a list (if not already
+        present).  Strings are written directly to the output document, e.g. for
+        added music functions or pieces of Ccheme code.  Functions are called on
+        output time and should return one ly.dom.Node object that is inserted in
+        the output document.
+        """
+        if not isinstance(code, (basestring, types.FunctionType)):
+            for c in code:
+                self.addCodeBlock(c)
+        elif code not in self.codeBlocks:
+            self.codeBlocks.append(code)
 
     def getInstrumentNames(self, names, num=0):
         """
@@ -920,9 +963,15 @@ class Builder(object):
                 names = self.getInstrumentNames(names, num)
             ly.dom.addInstrumentNameEngraverIfNecessary(node)
             w = node.getWith()
-            w['instrumentName'] = names[s.instrFirst.currentIndex()]
+            first = names[s.instrFirst.currentIndex()]
+            w['instrumentName'] = first
             if s.instrOther.currentIndex() < 2:
-                w['shortInstrumentName'] = names[s.instrOther.currentIndex()]
+                other = names[s.instrOther.currentIndex()]
+                # if these are markup objects, copy them otherwise the assignment
+                # to shortInstrumentName takes it away from the instrumentName.
+                if other is first and isinstance(first, ly.dom.Node):
+                    other = other.copy()
+                w['shortInstrumentName'] = other
 
     def setMidiInstrument(self, node, midiInstrument):
         """
@@ -1039,5 +1088,3 @@ a {
 durations = ['16', '16.', '8', '8.', '4', '4.', '2', '2.', '1', '1.']
 midiDurations = ((16,1),(32,3),(8,1),(16,3),(4,1),(8,3),(2,1),(4,3),(1,1),(2,3))
 partialDurations = ((4,0),(4,1),(3,0),(3,1),(2,0),(2,1),(1,0),(1,1),(0,0),(0,1))
-
-paperSizes = ['a3', 'a4', 'a5', 'a6', 'a7', 'legal', 'letter', '11x17']
