@@ -1,6 +1,6 @@
 # This file is part of the Frescobaldi project, http://www.frescobaldi.org/
 #
-# Copyright (c) 2008, 2009  Wilbert Berendsen
+# Copyright (c) 2008, 2009, 2010 by Wilbert Berendsen
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -22,10 +22,12 @@ Part types for the Score Wizard (scorewiz/__init__.py).
 In separate file to ease maintenance.
 """
 
-from PyQt4.QtCore import QObject, SIGNAL
+from rational import Rational
+
+from PyQt4.QtCore import QObject, QRegExp, Qt, SIGNAL
 from PyQt4.QtGui import (
-    QCheckBox, QComboBox, QGroupBox, QLabel, QRadioButton, QSpinBox,
-    QVBoxLayout)
+    QCheckBox, QComboBox, QGroupBox, QLabel, QRadioButton, QRegExpValidator,
+    QSpinBox, QVBoxLayout)
 from PyKDE4.kdecore import i18n, ki18n
 from PyKDE4.kdeui import KHBox, KVBox
 
@@ -35,62 +37,69 @@ import frescobaldi_app.scorewiz
 
 I18N_NOOP = lambda s: s
 
+
+# Widgets used by different part types
+
+def voicesWidget(
+    layout, title=None, minValue=1, maxValue=4, default=1, tooltip=None):
+    """
+    Creates a widget for setting the number of voices.
+    Adds a HBox to the layout and returns the created QSpinBox.
+    """
+    h = KHBox()
+    l = QLabel(title or i18n("Voices:"), h)
+    sb = QSpinBox(h)
+    sb.setRange(minValue, maxValue)
+    sb.setValue(default)
+    l.setBuddy(sb)
+    sb.setToolTip(tooltip or i18n("How many voices to put in this staff."))
+    layout.addWidget(h)
+    return sb
+
+
 # Base classes for the part types in this file.
 # (For the real part type classes see below.)
 
-
 class Part(frescobaldi_app.scorewiz.PartBase):
     """
     The base class for our part types.
     Adds some convenience methods for often used tasks.
     """
-    def assign(self, node, stub, nameref=None):
+    def assign(self, name=None):
         """
-        Creates an assignment for the stub under the given name.
-        Adds an identifier to the given node.
-        nameref can be a string name or a Reference object.
-
-        If the nameref is empty or None, the identifier() will be used,
-        with the first letter lowered.
+        Creates an assignment. name is a string name, if not given
+        the class name is used with the first letter lowered.
+        returns the assignment and the reference for the name.
         """
-        if not nameref:
-            nameref = Reference(self.identifier())
-        elif not isinstance(nameref, Reference):
-            nameref = Reference(nameref)
-        # handle multiple references to the same assignment
-        for a in self.assignments:
-            if a.name.name == nameref.name:
-                nameref = a.name
-                break
-        else:
-            a = Assignment(nameref)
-            a.append(stub)
-            self.assignments.append(a)
-        Identifier(nameref, node)
-
-    def assignMusic(self, node, octave, transpose=None, name=None):
+        ref = Reference(name or self.identifier())
+        a = Assignment(ref)
+        self.assignments.append(a)
+        return a, ref
+    
+    def assignMusic(self, name=None, octave=0, transposition=None):
         """
         Creates a \\relative stub and an assignment for it.
-        Returns the contents of the stub for other possible manipulations.
+        Returns the contents of the stub for other possible manipulations,
+        and the Reference object.
         """
-        stub = Relative()
+        a, ref = self.assign(name)
+        stub = Relative(a)
         Pitch(octave, 0, 0, stub)
         s = Seq(stub)
         Identifier('global', s).after = 1
-        if transpose is not None:
-            toct, tnote, talter = transpose
+        if transposition is not None:
+            toct, tnote, talter = transposition
             Pitch(toct, tnote, Rational(talter, 2), Transposition(s))
         LineComment(i18n("Music follows here."), s)
         BlankLine(s)
-        self.assign(node, stub, name)
-        return s
+        return s, ref
 
 
 class SingleVoicePart(Part):
     """
     The abstract base class for single voice part types.
     The build function just creates one staff with one voice,
-    and uses the .clef, .transpose, .midiInstrument and .instrumentNames
+    and uses the .clef, .transposition, .midiInstrument and .instrumentNames
     class (or instance) attributes.
     """
 
@@ -101,7 +110,7 @@ class SingleVoicePart(Part):
     octave = 1
 
     # A subclass could set a transposition here.
-    transpose = None
+    transposition = None
 
     # The MIDI instrument to use: see
     # http://lilypond.org/doc/latest/Documentation/user/lilypond/MIDI-instrument-names
@@ -117,13 +126,14 @@ class SingleVoicePart(Part):
         possible transposition.  Returns the staff and the stub for other
         possible manipulations.
         """
+        stub, ref = self.assignMusic(None, self.octave, self.transposition)
         staff = Staff()
         builder.setInstrumentNames(staff, self.instrumentNames, self.num)
         builder.setMidiInstrument(staff, self.midiInstrument)
-        s1 = braces and Seq(staff) or Seqr(staff)
+        s = braces and Seq(staff) or Seqr(staff)
         if self.clef:
-            Clef(self.clef, s1)
-        stub = self.assignMusic(s1, self.octave, self.transpose)
+            Clef(self.clef, s)
+        Identifier(ref, s)
         self.nodes.append(staff)
         return staff, stub
 
@@ -138,6 +148,7 @@ class StringPart(SingleVoicePart):
 class TablaturePart(SingleVoicePart):
     """
     A class for instruments that support TabStaffs.
+    Can easily support multi-voice staves.
     """
     octave = 0
     tunings = ()    # may contain a list of tunings.
@@ -176,33 +187,74 @@ class TablaturePart(SingleVoicePart):
         Non-zero if the user wants a TabStaff.
         """
         self.tuningSel.setEnabled(bool(enable))
-
+    
+    def voiceCount(self):
+        """
+        Returns the number of voices. Can be made user-settable in subclasses.
+        """
+        return 1
+        
     def build(self, builder):
-        t = self.staffType.currentIndex()
-        if t == 0:
-            # normal staff only
-            return super(TablaturePart, self).build(builder)
-
-        # make a tabstaff
-        tab = TabStaff()
-        if self.tabFormat:
-            tab.getWith()['tablatureFormat'] = Scheme(self.tabFormat)
-        self.setTunings(tab)
-        s = Seqr(tab)
-        ref = Reference(self.identifier())
-        self.assignMusic(s, self.octave, self.transpose, name=ref)
-        if t == 1:  # only a TabStaff
-            builder.setMidiInstrument(tab, self.midiInstrument)
-            p = tab
-        else:       # both TabStaff and normal staff
+        # First make assignments for the voices we want to create
+        numVoices = self.voiceCount()
+        if numVoices == 1:
+            voices = (self.identifier(),)
+        elif numVoices == 2:
+            order = 1, 2
+            voices = 'upper', 'lower'
+        elif numVoices == 3:
+            order = 1, 3, 2
+            voices = 'upper', 'middle', 'lower'
+        else:
+            order = 1, 2, 3, 4
+            voices = [self.identifier() + ly.nums(i) for i in order]
+        
+        refs = [self.assignMusic(name, self.octave, self.transposition)[1]
+                    for name in voices]
+        
+        staffType = self.staffType.currentIndex()
+        if staffType in (0, 2):
+            # create a normal staff
+            staff = Staff()
+            seq = Seqr(staff)
+            if self.clef:
+                Clef(self.clef, seq)
+            mus = Simr(seq)
+            for ref in refs[:-1]:
+                Identifier(ref, mus)
+                VoiceSeparator(mus)
+            Identifier(refs[-1], mus)
+            builder.setMidiInstrument(staff, self.midiInstrument)
+        
+        if staffType in (1, 2):
+            # create a tab staff
+            tabstaff = TabStaff()
+            if self.tabFormat:
+                tabstaff.getWith()['tablatureFormat'] = Scheme(self.tabFormat)
+            self.setTunings(tabstaff)
+            sim = Simr(tabstaff)
+            if numVoices == 1:
+                Identifier(refs[0], sim)
+            else:
+                for num, ref in zip(order, refs):
+                    s = Seq(TabVoice(parent=sim))
+                    Text('\\voice%s' % ly.nums(num), s)
+                    Identifier(ref, s)
+        
+        if staffType == 0:
+            # only a normal staff
+            p = staff
+        elif staffType == 1:
+            # only a TabStaff
+            builder.setMidiInstrument(tabstaff, self.midiInstrument)
+            p = tabstaff
+        else:
+            # both TabStaff and normal staff
             p = StaffGroup()
             s = Sim(p)
-            m = Seqr(Staff(parent=s))
-            s.append(tab)
-            builder.setMidiInstrument(m.parent(), self.midiInstrument)
-            if self.clef:
-                Clef(self.clef, m)
-            Identifier(ref, m)
+            s.append(staff)
+            s.append(tabstaff)
+        
         builder.setInstrumentNames(p, self.instrumentNames, self.num)
         self.nodes.append(p)
 
@@ -231,16 +283,22 @@ class VocalPart(Part):
     Base class for vocal stuff.
     """
     midiInstrument = 'choir aahs'
-
-    def assignLyrics(self, node, name, verse = 0):
+    
+    def assignLyrics(self, name, verse=0):
+        """
+        Creates an empty assignment for lyrics.
+        Returns the reference for the name.
+        """
         l = LyricMode()
         if verse:
             name = name + ly.nums(verse)
             Line('\\set stanza = "%d."' % verse, l)
+        a, ref = self.assign(name)
+        a.append(l)
         LineComment(i18n("Lyrics follow here."), l)
         BlankLine(l)
-        self.assign(node, l, name)
-
+        return ref
+        
     def widgets(self, layout):
         self.stanzaWidget(layout)
         self.ambitusWidget(layout)
@@ -262,14 +320,14 @@ class VocalPart(Part):
 
     def addStanzas(self, node):
         """
-        Add stanzas in self.stanzas.value() to the (Voice) node
+        Add stanzas in self.stanzas.value() to the given (Voice) node
         using \\addlyrics.
         """
         if self.stanzas.value() == 1:
-            self.assignLyrics(AddLyrics(node), 'verse')
+            Identifier(self.assignLyrics('verse'), AddLyrics(node))
         else:
             for i in range(self.stanzas.value()):
-                self.assignLyrics(AddLyrics(node), 'verse', i + 1)
+                Identifier(self.assignLyrics('verse', i + 1), AddLyrics(node))
 
 
 class VocalSoloPart(VocalPart, SingleVoicePart):
@@ -298,13 +356,16 @@ class KeyboardPart(Part):
         if clef:
             Clef(clef, c)
         if numVoices == 1:
-            self.assignMusic(c, octave, name=name)
+            stub, ref = self.assignMusic(name, octave)
+            Identifier(ref, c)
         else:
             c = Sim(c)
             for i in range(1, numVoices):
-                self.assignMusic(c, octave, name=name + ly.nums(i))
+                stub, ref = self.assignMusic(name + ly.nums(i), octave)
+                Identifier(ref, c)
                 VoiceSeparator(c)
-            self.assignMusic(c, octave, name=name + ly.nums(numVoices))
+            stub, ref = self.assignMusic(name + ly.nums(numVoices), octave)
+            Identifier(ref, c)
         return staff
 
     def build(self, builder):
@@ -313,8 +374,8 @@ class KeyboardPart(Part):
         builder.setInstrumentNames(p, self.instrumentNames, self.num)
         s = Sim(p)
         # add two staves, with a respective number of voices.
-        self.buildStaff(builder, 'right', 1, self.rightVoices.value(), s)
-        self.buildStaff(builder, 'left', 0, self.leftVoices.value(), s, "bass")
+        self.buildStaff(builder, 'right', 1, self.upperVoices.value(), s)
+        self.buildStaff(builder, 'left', 0, self.lowerVoices.value(), s, "bass")
         self.nodes.append(p)
 
     def widgets(self, layout):
@@ -324,18 +385,8 @@ class KeyboardPart(Part):
             "like a fuge.")))
         l.setWordWrap(True)
         layout.addWidget(l)
-        h = KHBox()
-        layout.addWidget(h)
-        l = QLabel(i18n("Right hand:"), h)
-        self.rightVoices = QSpinBox(h)
-        self.rightVoices.setRange(1, 4)
-        l.setBuddy(self.rightVoices)
-        h = KHBox()
-        layout.addWidget(h)
-        l = QLabel(i18n("Left hand:"), h)
-        self.leftVoices = QSpinBox(h)
-        self.leftVoices.setRange(1, 4)
-        l.setBuddy(self.leftVoices)
+        self.upperVoices = voicesWidget(layout, i18n("Right hand:"))
+        self.lowerVoices = voicesWidget(layout, i18n("Left hand:"))
 
 
 class PitchedPercussionPart(SingleVoicePart):
@@ -358,8 +409,9 @@ class Chords(Part):
 
     def build(self, builder):
         p = ChordNames()
-        s = ChordMode()
-        name = Reference('chordNames')
+        a, ref = self.assign('chordNames')
+        Identifier(ref, p)
+        s = ChordMode(a)
         Identifier('global', s).after = 1
         i = self.chordStyle.currentIndex()
         if i > 0:
@@ -367,11 +419,10 @@ class Chords(Part):
                 ('german', 'semiGerman', 'italian', 'french')[i-1], s)
         LineComment(i18n("Chords follow here."), s)
         BlankLine(s)
-        self.assign(p, s, name)
         self.nodes.append(p)
         if self.guitarFrets.isChecked():
             f = FretBoards()
-            Identifier(name, f)
+            Identifier(ref, f)
             self.nodes.append(f)
             builder.include("predefined-guitar-fretboards.ly")
 
@@ -398,12 +449,13 @@ class BassFigures(Part):
     _name = ki18n("Figured Bass")
 
     def build(self, builder):
+        a, ref = self.assign('figBass')
+        s = FigureMode(a)
         p = FiguredBass()
-        s = FigureMode()
+        Identifier(ref, p)
         Identifier('global', s)
         LineComment(i18n("Figures follow here."), s)
         BlankLine(s)
-        self.assign(p, s, 'figBass')
         if self.useExtenderLines.isChecked():
             p.getWith()['useBassFigureExtenders'] = Scheme('#t')
         self.nodes.append(p)
@@ -453,14 +505,16 @@ class BassoContinuo(Cello):
         builder.setMidiInstrument(p, self.midiInstrument)
         s = Sim(p)
         Clef("bass", s)
-        self.assignMusic(s, self.octave, self.transpose, 'bcMusic')
-        b = FigureMode()
+        stub, ref = self.assignMusic('bcMusic', self.octave, self.transposition)
+        Identifier(ref, s)
+        a, ref = self.assign('bcFigures')
+        b = FigureMode(a)
+        Identifier(ref, s)
         Identifier('global', b)
         Line("\\override Staff.BassFigureAlignmentPositioning "
              "#'direction = #DOWN", b)
         LineComment(i18n("Figures follow here."), b)
         BlankLine(b)
-        self.assign(s, b, 'bcFigures')
         self.nodes.append(p)
 
 
@@ -513,6 +567,13 @@ class ClassicalGuitar(TablaturePart):
         (ki18n("Open G-tuning"), 'guitar-open-g-tuning'),
     )
 
+    def widgets(self, layout):
+        super(ClassicalGuitar, self).widgets(layout)
+        self.numVoices = voicesWidget(layout)
+        
+    def voiceCount(self):
+        return self.numVoices.value()
+
 
 class JazzGuitar(ClassicalGuitar):
     _name = ki18n("Jazz guitar")
@@ -543,18 +604,19 @@ class Harp(KeyboardPart):
     midiInstrument = 'harp'
 
     def build(self, builder):
-        """ setup structure for 2 staves. """
+        """ setup structure for a 2-staff PianoStaff. """
         p = PianoStaff()
         builder.setInstrumentNames(p, self.instrumentNames, self.num)
         s = Sim(p)
-        # add two staves, with one voice each.
-        self.buildStaff(builder, 'upper', 1, 1, s)
-        self.buildStaff(builder, 'lower', 0, 1, s, "bass")
+        # add two staves, with a respective number of voices.
+        self.buildStaff(builder, 'upper', 1, self.upperVoices.value(), s)
+        self.buildStaff(builder, 'lower', 0, self.lowerVoices.value(), s, "bass")
         self.nodes.append(p)
 
     def widgets(self, layout):
-        """ don't display the voice widgets of the KeyboardPart """
-        super(KeyboardPart, self).widgets(layout)
+        super(Harp, self).widgets(layout)
+        self.upperVoices.parent().findChild(QLabel).setText(i18n("Upper staff:"))
+        self.lowerVoices.parent().findChild(QLabel).setText(i18n("Lower staff:"))
 
 
 class Flute(WoodWindPart):
@@ -567,14 +629,14 @@ class Piccolo(WoodWindPart):
     _name = ki18n("Piccolo")
     instrumentNames = I18N_NOOP("Piccolo|Pic.")
     midiInstrument = 'piccolo'
-    transpose = (1, 0, 0)
+    transposition = (1, 0, 0)
 
 
 class BassFlute(WoodWindPart):
     _name = ki18n("Bass flute")
     instrumentNames = I18N_NOOP("Bass flute|Bfl.")
     midiInstrument = 'flute'
-    transpose = (-1, 4, 0)
+    transposition = (-1, 4, 0)
 
 
 class Oboe(WoodWindPart):
@@ -587,14 +649,14 @@ class OboeDAmore(WoodWindPart):
     _name = ki18n("Oboe d'Amore")
     instrumentNames = I18N_NOOP("Oboe d'amore|Ob.d'am.")
     midiInstrument = 'oboe'
-    transpose = (-1, 5, 0)
+    transposition = (-1, 5, 0)
 
 
 class EnglishHorn(WoodWindPart):
     _name = ki18n("English Horn")
     instrumentNames = I18N_NOOP("English horn|Eng.h.")
     midiInstrument = 'english horn'
-    transpose = (-1, 3, 0)
+    transposition = (-1, 3, 0)
 
 
 class Bassoon(WoodWindPart):
@@ -609,7 +671,7 @@ class ContraBassoon(WoodWindPart):
     _name = ki18n("Contrabassoon")
     instrumentNames = I18N_NOOP("Contrabassoon|C.Bn.")
     midiInstrument = 'bassoon'
-    transpose = (-1, 0, 0)
+    transposition = (-1, 0, 0)
     clef = 'bass'
     octave = -1
 
@@ -618,56 +680,56 @@ class Clarinet(WoodWindPart):
     _name = ki18n("Clarinet")
     instrumentNames = I18N_NOOP("Clarinet|Cl.")
     midiInstrument = 'clarinet'
-    transpose = (-1, 6, -1)
+    transposition = (-1, 6, -1)
 
 
 class SopraninoSax(WoodWindPart):
     _name = ki18n("Sopranino Sax")
     instrumentNames = I18N_NOOP("Sopranino Sax|SiSx.")
     midiInstrument = 'soprano sax'
-    transpose = (0, 2, -1)    # es'
+    transposition = (0, 2, -1)    # es'
 
 
 class SopranoSax(WoodWindPart):
     _name = ki18n("Soprano Sax")
     instrumentNames = I18N_NOOP("Soprano Sax|SoSx.")
     midiInstrument = 'soprano sax'
-    transpose = (-1, 6, -1)   # bes
+    transposition = (-1, 6, -1)   # bes
 
 
 class AltoSax(WoodWindPart):
     _name = ki18n("Alto Sax")
     instrumentNames = I18N_NOOP("Alto Sax|ASx.")
     midiInstrument = 'alto sax'
-    transpose = (-1, 2, -1)   # es
+    transposition = (-1, 2, -1)   # es
 
 
 class TenorSax(WoodWindPart):
     _name = ki18n("Tenor Sax")
     instrumentNames = I18N_NOOP("Tenor Sax|TSx.")
     midiInstrument = 'tenor sax'
-    transpose = (-2, 6, -1)   # bes,
+    transposition = (-2, 6, -1)   # bes,
 
 
 class BaritoneSax(WoodWindPart):
     _name = ki18n("Baritone Sax")
     instrumentNames = I18N_NOOP("Baritone Sax|BSx.")
     midiInstrument = 'baritone sax'
-    transpose = (-2, 2, -1)   # es,
+    transposition = (-2, 2, -1)   # es,
 
 
 class BassSax(WoodWindPart):
     _name = ki18n("Bass Sax")
     instrumentNames = I18N_NOOP("Bass Sax|BsSx.")
     midiInstrument = 'baritone sax'
-    transpose = (-3, 6, -1)   # bes,,
+    transposition = (-3, 6, -1)   # bes,,
 
 
 class SopranoRecorder(WoodWindPart):
     _name = ki18n("Soprano recorder")
     instrumentNames = I18N_NOOP("Soprano recorder|S.rec.")
     midiInstrument = 'recorder'
-    transpose = (1, 0, 0)
+    transposition = (1, 0, 0)
 
 
 class AltoRecorder(WoodWindPart):
@@ -694,7 +756,7 @@ class HornF(BrassPart):
     _name = ki18n("Horn in F")
     instrumentNames = I18N_NOOP("Horn in F|Hn.F.")
     midiInstrument = 'french horn'
-    transpose = (-1, 3, 0)
+    transposition = (-1, 3, 0)
 
 
 class TrumpetC(BrassPart):
@@ -706,7 +768,7 @@ class TrumpetC(BrassPart):
 class TrumpetBb(TrumpetC):
     _name = ki18n("Trumpet in Bb")
     instrumentNames = I18N_NOOP("Trumpet in Bb|Tr.Bb")
-    transpose = (-1, 6, -1)
+    transposition = (-1, 6, -1)
 
 
 class Trombone(BrassPart):
@@ -721,14 +783,14 @@ class Tuba(BrassPart):
     _name = ki18n("Tuba")
     instrumentNames = I18N_NOOP("Tuba|Tb.")
     midiInstrument = 'tuba'
-    transpose = (-2, 6, -1)
+    transposition = (-2, 6, -1)
 
 
 class BassTuba(BrassPart):
     _name = ki18n("Bass Tuba")
     instrumentNames = I18N_NOOP("Bass Tuba|B.Tb.")
     midiInstrument = 'tuba'
-    transpose = (-2, 0, 0)
+    transposition = (-2, 0, 0)
     clef = 'bass'
     octave = -1
 
@@ -781,14 +843,17 @@ class LeadSheet(VocalPart, Chords):
             mel = Sim(Staff(parent=s))
             v1 = Voice(parent=mel)
             s1 = Seq(v1)
-            Line('\\voiceOne', s1)
-            self.assignMusic(s1, 1, name='melody')
+            Text('\\voiceOne', s1)
+            stub, ref = self.assignMusic('melody', 1)
+            Identifier(ref, s1)
             s2 = Seq(Voice(parent=mel))
-            Line('\\voiceTwo', s2)
-            self.assignMusic(s2, 0, name='accRight')
+            Text('\\voiceTwo', s2)
+            stub, ref = self.assignMusic('accRight', 0)
+            Identifier(ref, s2)
             acc = Seq(Staff(parent=s))
             Clef('bass', acc)
-            self.assignMusic(acc, -1, name='accLeft')
+            stub, ref = self.assignMusic('accLeft', -1)
+            Identifier(ref, acc)
             if self.ambitus.isChecked():
                 # We can't use \addlyrics when the voice has a \with {}
                 # section, because it creates a nested Voice context.
@@ -802,17 +867,20 @@ class LeadSheet(VocalPart, Chords):
                 if count == 1:
                     l = Lyrics()
                     s.insert(acc.parent(), l)
-                    self.assignLyrics(LyricsTo(v1.cid, l), 'verse')
+                    ref = self.assignLyrics('verse')
+                    Identifier(ref, LyricsTo(v1.cid, l))
                 else:
                     for i in range(count):
                         l = Lyrics()
                         s.insert(acc.parent(), l)
-                        self.assignLyrics(LyricsTo(v1.cid, l), 'verse', i + 1)
+                        ref = self.assignLyrics('verse', i + 1)
+                        Identifier(ref, LyricsTo(v1.cid, l))
             else:
                 self.addStanzas(v1)
         else:
+            stub, ref = self.assignMusic('melody', 1)
             p = Staff()
-            self.assignMusic(Seq(p), 1, name='melody')
+            Identifier(ref, Seq(p))
             self.addStanzas(p)
             if self.ambitus.isChecked():
                 Line('\\consists "Ambitus_engraver"', p.getWith())
@@ -843,7 +911,7 @@ class Choir(VocalPart):
     _name = ki18n("Choir")
 
     def widgets(self, layout):
-        l = QLabel('<p>%s</p><p><i>(%s)</i></p>' % (
+        l = QLabel('<p>%s <i>(%s)</i></p>' % (
             i18n("Please select the voices for the choir. "
             "Use the letters S, A, T, or B. A hyphen denotes a new staff."),
             i18n("Hint: For a double choir you can use two choir parts.")))
@@ -855,6 +923,9 @@ class Choir(VocalPart):
         self.voicing = QComboBox(h)
         l.setBuddy(self.voicing)
         self.voicing.setEditable(True)
+        self.voicing.setCompleter(None)
+        self.voicing.setValidator(QRegExpValidator(
+            QRegExp("[SATB]+(-[SATB]+)*", Qt.CaseInsensitive), self.voicing))
         self.voicing.addItems((
             'SA-TB', 'S-A-T-B',
             'SA', 'S-A', 'SS-A', 'S-S-A',
@@ -882,6 +953,10 @@ class Choir(VocalPart):
         group.addWidget(self.lyrEachDiff)
         self.stanzaWidget(group)
         self.ambitusWidget(layout)
+        self.pianoReduction = QCheckBox(i18n("Piano reduction"))
+        self.pianoReduction.setToolTip(i18n(
+            "Adds an automatically generated piano reduction."))
+        layout.addWidget(self.pianoReduction)
 
     partInfo = {
         'S': ('soprano', 1, SopranoVoice.instrumentNames),
@@ -897,6 +972,8 @@ class Choir(VocalPart):
         staves = re.sub(r'[^SATB-]+', '', staves)
         # remove double hyphens, and from begin and end
         staves = re.sub('-+', '-', staves).strip('-')
+        if not staves:
+            return
         splitStaves = staves.split('-')
         p = ChoirStaff()
         choir = Sim(p)
@@ -905,9 +982,10 @@ class Choir(VocalPart):
         if len(splitStaves) > 1 and self.num:
             builder.setInstrumentNames(p, I18N_NOOP("Choir|Ch."), self.num)
         count = dict.fromkeys('SATB', 0)  # dict with count of parts.
-        toGo = len(splitStaves)
         maxLen = max(map(len, splitStaves))
+        toGo = max(2, len(splitStaves)) # nr staves with same lyrics below + 1
         lyr, staffNames = [], []
+        pianoReduction = dict((key, []) for key in 'SATB')
         for staff in splitStaves:
             toGo -= 1
             # sort the letters in order SATB
@@ -922,7 +1000,7 @@ class Choir(VocalPart):
                     count[part] += 1
                 name, octave, instrName = self.partInfo[part]
                 instrNames.append((instrName, count[part]))
-                voices.append((name, count[part], octave))
+                voices.append((part, name, count[part], octave))
             if len(staff) == 1:
                 # There is only one voice in the staff. Just set the instrument
                 # name directly in the staff.
@@ -959,7 +1037,7 @@ class Choir(VocalPart):
 
             # Add the voices
             if len(staff) == 1:
-                name, num, octave = voices[0]
+                part, name, num, octave = voices[0]
                 mname = name + (num and ly.nums(num) or '')
                 if self.lyrEachDiff.isChecked():
                     lyrName = mname + 'Verse'
@@ -967,7 +1045,8 @@ class Choir(VocalPart):
                     lyrName = 'verse'
                 if maxLen == 1:
                     # if all staves have only one voice, use \addlyrics...
-                    self.assignMusic(mus, octave, name=mname)
+                    stub, ref = self.assignMusic(mname, octave)
+                    Identifier(ref, mus)
                     if not (self.lyrAllSame.isChecked() and not toGo):
                         for verse in stanzas:
                             lyr.append((AddLyrics(s), lyrName, verse))
@@ -975,12 +1054,13 @@ class Choir(VocalPart):
                     # otherwise create explicit Voice and Lyrics contexts.
                     vname = name + str(num or '')
                     v = Seqr(Voice(vname, parent=mus))
-                    self.assignMusic(v, octave, name=mname)
+                    stub, ref = self.assignMusic(mname, octave)
+                    Identifier(ref, v)
                     if not (self.lyrAllSame.isChecked() and not toGo):
                         for verse in stanzas:
                             lyr.append((LyricsTo(vname, Lyrics(parent=choir)),
                                 lyrName, verse))
-
+                pianoReduction[part].append(ref)
                 if self.ambitus.isChecked():
                     Line('\\consists "Ambitus_engraver"', s.getWith())
             else:
@@ -1007,7 +1087,7 @@ class Choir(VocalPart):
                 # We want the staff name (actually context-id) in lower case.
                 staffName = staffName.lower()
                 # Create the voices and their lyrics.
-                for (name, num, octave), vnum in zip(voices, order):
+                for (part, name, num, octave), vnum in zip(voices, order):
                     mname = name + (num and ly.nums(num) or '')
                     vname = name + str(num or '')
                     v = Voice(vname, parent=mus)
@@ -1019,7 +1099,9 @@ class Choir(VocalPart):
                                  ((vnum - 1) * 2.0), v.getWith())
                     v = Seq(v)
                     Text('\\voice' + ly.nums(vnum), v)
-                    self.assignMusic(v, octave, name=mname)
+                    stub, ref = self.assignMusic(mname, octave)
+                    Identifier(ref, v)
+                    pianoReduction[part].append(ref)
                     if self.lyrAllSame.isChecked() and toGo and vnum == 1:
                         lyrName = 'verse'
                         above = False
@@ -1043,10 +1125,75 @@ class Choir(VocalPart):
                         lyr.append((LyricsTo(vname, l), lyrName, verse))
 
         # Assign the lyrics, so their definitions come after the note defs.
+        refs = {}
         for node, name, verse in lyr:
-            self.assignLyrics(node, name, verse)
+            if (name, verse) not in refs:
+                refs[(name, verse)] = self.assignLyrics(name, verse)
+            Identifier(refs[(name, verse)], node)
         self.nodes.append(p)
 
+        # Create the piano reduction if desired
+        if self.pianoReduction.isChecked():
+            a, ref = self.assign('pianoReduction')
+            self.nodes.append(Identifier(ref))
+            piano = PianoStaff(parent=a)
+            
+            sim = Sim(piano)
+            rightStaff = Staff(parent=sim)
+            leftStaff = Staff(parent=sim)
+            right = Seq(rightStaff)
+            left = Seq(leftStaff)
+            
+            # Determine the ordering of voices in the staves
+            upper = pianoReduction['S'] + pianoReduction['A']
+            lower = pianoReduction['T'] + pianoReduction['B']
+            
+            preferUpper = 1
+            if not upper:
+                # Male choir
+                upper = pianoReduction['T']
+                lower = pianoReduction['B']
+                Clef("treble_8", right)
+                Clef("bass", left)
+                preferUpper = 0
+            elif not lower:
+                # Female choir
+                upper = pianoReduction['S']
+                lower = pianoReduction['A']
+            else:
+                Clef("bass", left)
+
+            # Otherwise accidentals can be confusing
+            Line("#(set-accidental-style 'piano)", right)
+            Line("#(set-accidental-style 'piano)", left)
+            
+            # Move voices if unevenly spread
+            if abs(len(upper) - len(lower)) > 1:
+                voices = upper + lower
+                half = (len(voices) + preferUpper) / 2
+                upper = voices[:half]
+                lower = voices[half:]
+            
+            for staff, voices in (Simr(right), upper), (Simr(left), lower):
+                if voices:
+                    for v in voices[:-1]:
+                        Identifier(v, staff)
+                        VoiceSeparator(staff).after = 1
+                    Identifier(voices[-1], staff)
+
+            # Make the piano part somewhat smaller
+            Line("fontSize = #-1", piano.getWith())
+            Line("\\override StaffSymbol #'staff-space = #(magstep -1)", piano.getWith())
+            
+            # Nice to add Mark engravers
+            Line('\\consists "Mark_engraver"', rightStaff.getWith())
+            Line('\\consists "Metronome_mark_engraver"', rightStaff.getWith())
+            
+            # Keep piano reduction out of the MIDI output
+            if builder.midi:
+                Line('\\remove "Staff_performer"', rightStaff.getWith())
+                Line('\\remove "Staff_performer"', leftStaff.getWith())
+
 
 class Piano(KeyboardPart):
     _name = ki18n("Piano")
@@ -1073,16 +1220,8 @@ class Organ(KeyboardPart):
 
     def widgets(self, layout):
         super(Organ, self).widgets(layout)
-        h = KHBox()
-        layout.addWidget(h)
-        l = QLabel(i18n("Pedal:"), h)
-        self.pedalVoices = QSpinBox(h)
-        self.pedalVoices.setMinimum(0)
-        self.pedalVoices.setMaximum(4)
-        l.setBuddy(self.pedalVoices)
-        self.pedalVoices.setValue(1)
-        self.pedalVoices.setToolTip(i18n(
-            "Set to 0 to disable the pedal altogether."))
+        self.pedalVoices = voicesWidget(layout, i18n("Pedal:"), 0,
+            tooltip=i18n("Set to 0 to disable the pedal altogether."))
 
     def build(self, builder):
         super(Organ, self).build(builder)
@@ -1139,12 +1278,17 @@ class Drums(Part):
     _name = ki18n("Drums")
     instrumentNames = I18N_NOOP("Drums|Dr.")
 
-    def assignDrums(self, node, name):
-        s = DrumMode()
+    def assignDrums(self, name = None):
+        """
+        Creates an empty name = \drummode assignment and returns
+        the reference for the name.
+        """
+        a, ref = self.assign(name)
+        s = DrumMode(a)
         Identifier('global', s)
         LineComment(i18n("Drums follow here."), s)
         BlankLine(s)
-        self.assign(node, s, name)
+        return ref
 
     def build(self, builder):
         p = DrumStaff()
@@ -1153,9 +1297,11 @@ class Drums(Part):
             for i in range(1, self.drumVoices.value()+1):
                 q = Seq(DrumVoice(parent=s))
                 Text('\\voice%s' % ly.nums(i), q)
-                self.assignDrums(q, 'drum%s' % ly.nums(i))
+                ref = self.assignDrums('drum%s' % ly.nums(i))
+                Identifier(ref, q)
         else:
-            self.assignDrums(s, 'drum')
+            ref = self.assignDrums('drum')
+            Identifier(ref, s)
         builder.setInstrumentNames(p, self.instrumentNames, self.num)
         i = self.drumStyle.currentIndex()
         if i > 0:
@@ -1170,14 +1316,7 @@ class Drums(Part):
         self.nodes.append(p)
 
     def widgets(self, layout):
-        h = KHBox()
-        layout.addWidget(h)
-        l = QLabel(i18n("Voices:"), h)
-        self.drumVoices = QSpinBox(h)
-        self.drumVoices.setMinimum(1)
-        self.drumVoices.setMaximum(4)
-        l.setBuddy(self.drumVoices)
-        h.setToolTip(i18n("How many drum voices to put in this staff."))
+        self.drumVoices = voicesWidget(layout)
         h = KHBox()
         layout.addWidget(h)
         l = QLabel(i18n("Style:"), h)
