@@ -1,6 +1,6 @@
 # This file is part of the Frescobaldi project, http://www.frescobaldi.org/
 #
-# Copyright (c) 2008, 2009  Wilbert Berendsen
+# Copyright (c) 2008, 2009, 2010 by Wilbert Berendsen
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,14 +17,15 @@
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 # See http://www.gnu.org/licenses/ for more information.
 
-import os, re, sip, sys, weakref, dbus, dbus.service, dbus.mainloop.qt
+import os, re, sip, sys, time, weakref, dbus, dbus.service, dbus.mainloop.qt
 from dbus.service import method, signal
 
 from signals import Signal
 
 from PyQt4.QtCore import QObject, Qt, QVariant, SIGNAL
 from PyKDE4.kdecore import i18n, KConfig, KGlobal, KUrl
-from PyKDE4.kdeui import KApplication, KGuiItem, KMessageBox, KStandardGuiItem
+from PyKDE4.kdeui import (
+    KActionCollection, KApplication, KGuiItem, KMessageBox, KStandardGuiItem)
 from PyKDE4.kio import KEncodingFileDialog
 from PyKDE4.ktexteditor import KTextEditor
 
@@ -113,6 +114,10 @@ class MainApp(DBusItem):
 
         # restore session etc.
         
+    @lazymethod
+    def stateManager(self):
+        return StateManager(self)
+
     def defaultDirectory(self):
         return ''
 
@@ -124,8 +129,14 @@ class MainApp(DBusItem):
         
     def findDocument(self, url):
         """ Return the opened document or False. """
+        if not isinstance(url, KUrl):
+            url = KUrl(url)
+        # we use string comparison, because sometimes percent encoding
+        # issues make the same QUrls look different, esp when dragged
+        # from KMail...
+        url = url.toString()
         for d in self.documents:
-            if d.url() == url:
+            if d.url().toString() == url:
                 return d
         return False
     
@@ -133,18 +144,24 @@ class MainApp(DBusItem):
     def openUrl(self, url, encoding=None):
         if not isinstance(url, KUrl):
             url = KUrl(url)
+        # If no encoding given, set default or check if we can remember it
         if not encoding:
             encoding = self.defaultEncoding
+            if self.keepMetaInfo() and not url.isEmpty():
+                group = self.stateManager().groupForUrl(url)
+                if group:
+                    encoding = group.readEntry("encoding", QVariant('')).toString()
         # If there is only one document open and it is empty, nameless and
-        # unmodified, close it.
-        close0 = (not url.isEmpty() and len(self.documents) == 1
+        # unmodified, use it.
+        if (not url.isEmpty()
+            and len(self.documents) == 1
             and not self.documents[0].isModified()
-            and self.documents[0].url().isEmpty()
-            and self.documents[0].isEmpty())
-        d = (not url.isEmpty() and self.findDocument(url)
-            or self.createDocument(url, encoding))
-        if close0:
-            self.documents[0].close()
+            and self.documents[0].url().isEmpty()):
+            d = self.documents[0]
+            d.openUrl(url, encoding)
+        else:
+            d = (not url.isEmpty() and self.findDocument(url)
+                 or self.createDocument(url, encoding))
         return d
 
     @method(iface, in_signature='', out_signature='o')
@@ -217,7 +234,6 @@ class MainApp(DBusItem):
                 self.history[-2].setActive()
             self.documents.remove(doc)
             self.history.remove(doc)
-            self.documentClosed(doc)
             # Create empty document if last closed
             self.documents or self.createDocument().setActive()
 
@@ -248,6 +264,16 @@ class MainApp(DBusItem):
         import kateshell.exception
         kateshell.exception.showException(self, exctype, excvalue, exctb)
 
+    def keepMetaInfo(self):
+        """
+        Returns whether meta information about documents should be kept
+        (e.g. state of view, cursor position, encoding etc.)
+        
+        The default is to return False, reimplement this to return e.g.
+        a user's configured setting.
+        """
+        return False
+
 
 class Document(DBusItem):
     """
@@ -301,7 +327,6 @@ class Document(DBusItem):
         self.doc = self.app.editor.createDocument(self.app.mainwin)
         self.doc.setEncoding(self._encoding)
         self.view = self.doc.createView(self.app.mainwin)
-
         if not self._url.isEmpty():
             self.doc.openUrl(self._url)
         elif self.app.defaultMode:
@@ -334,6 +359,10 @@ class Document(DBusItem):
         for s in ("selectionChanged(KTextEditor::View*)",):
             QObject.connect(self.view, SIGNAL(s), self.slotSelectionChanged)
         
+        # Allow for dropping urls on the view
+        QObject.connect(self.view, SIGNAL("dropEventPass(QDropEvent *)"),
+            self.app.mainwin.dropEvent)
+        
         # delete some actions from the view before plugging in GUI
         # trick found in kateviewmanager.cpp
         for name in "file_save", "file_save_as":
@@ -343,6 +372,10 @@ class Document(DBusItem):
         
         # set default context menu
         self.view.setContextMenu(self.contextMenu())
+        # read state information (view settings etc.)
+        if self.app.keepMetaInfo():
+            self.app.stateManager().loadState(self)
+        # Let the world know ...
         self.app.documentMaterialized(self)
         self.viewCreated()
         
@@ -381,6 +414,19 @@ class Document(DBusItem):
         """
         pass 
     
+    def openUrl(self, url, encoding):
+        """
+        Opens a different url and re-init some stuff.
+        (Only call this on a Document that has already materialized.)
+        """
+        self.materialize()
+        self.setEncoding(encoding)
+        if not self.doc.openUrl(url):
+            self.slotDocumentUrlChanged() # otherwise signal is not emitted
+        if self.app.keepMetaInfo():
+            self.app.stateManager().loadState(self)
+        self.resetCursorTranslations()
+        
     @method(iface, in_signature='', out_signature='b')
     def save(self):
         if self.doc:
@@ -393,32 +439,42 @@ class Document(DBusItem):
     @method(iface, in_signature='', out_signature='b')
     def saveAs(self):
         if self.doc:
-            res = KEncodingFileDialog.getSaveUrlAndEncoding(
+            dlg = KEncodingFileDialog(
+                self.app.defaultDirectory(),
                 self.doc.encoding(),
-                self.url().url() or self.app.defaultDirectory(),
                 '\n'.join(self.app.fileTypes + ["*|%s" % i18n("All Files")]),
-                self.app.mainwin, i18n("Save File"))
-            if not res.URLs:
-                return False
-            url = res.URLs[0]
-            if (url.isLocalFile() and os.path.exists(unicode(url.path())) and
-                    KMessageBox.warningContinueCancel(self.app.mainwin,
+                i18n("Save File"),
+                KEncodingFileDialog.Saving,
+                self.app.mainwin)
+            dlg.setSelection(self.url().url())
+            
+            # because KEncodingFileDialog.setConfirmOverwrite() is not available
+            # in KDE 4.1, we try to use it, but fall back to our old own
+            # implementation.
+            checkOverwrite = False
+            try:
+                dlg.setConfirmOverwrite(True)
+            except AttributeError:
+                checkOverwrite = True
+            if not dlg.exec_():
+                return False # Cancelled
+            url = dlg.selectedUrl()
+            
+            # now confirm overwrite if the dialog didn't handle that (KDE < 4.2)
+            if (checkOverwrite and url.isLocalFile()
+                and os.path.exists(unicode(url.path()))
+                and KMessageBox.warningContinueCancel(self.app.mainwin,
                     i18n("A file named \"%1\" already exists. "
                          "Are you sure you want to overwrite it?",
                          url.fileName()),
                     i18n("Overwrite File?"), KGuiItem(i18n("&Overwrite"))) ==
                     KMessageBox.Cancel):
                 return False
+            
+            self.doc.setEncoding(dlg.selectedEncoding())
             return self.doc.saveAs(url)
         return True
             
-    def openUrl(self, url):
-        if not isinstance(url, KUrl):
-            url = KUrl(url)
-        self._url = url
-        if self.doc:
-            self.doc.openUrl(url)
-    
     @method(iface, in_signature='s', out_signature='')
     def setEncoding(self, encoding):
         if self.doc:
@@ -493,6 +549,7 @@ class Document(DBusItem):
             self.materialize()
             self.app.activeDocumentChanged(self)
         self.view.setFocus()
+        self.view.activateWindow()
 
     @method(iface, in_signature='iib', out_signature='')
     def setCursorPosition(self, line, column, translate=True):
@@ -563,6 +620,8 @@ class Document(DBusItem):
                 return False
             if not self.doc.closeUrl(False):
                 return False # closing did not succeed, but that'd be abnormal
+            if self.app.keepMetaInfo():
+                self.app.stateManager().saveState(self)
         self.closed(self) # before we are really deleted
         self.aboutToClose()
         self.app.documentClosed(self)
@@ -665,7 +724,9 @@ class Document(DBusItem):
                 m = markiface.mark(line)
                 if m:
                     marks.append("%d:%d" % (line, m))
-            group.writeEntry("bookmarks", ','.join(marks))
+            group.writeEntry("bookmarks", QVariant(','.join(marks)))
+        # also save the encoding
+        group.writeEntry("encoding", QVariant(self.encoding()))
 
     def line(self, lineNumber = None):
         """
@@ -697,7 +758,24 @@ class Document(DBusItem):
         """
         if self.view and self.view.selection():
             return unicode(self.view.selectionText())
-            
+    
+    def selectionOrDocument(self):
+        """
+        Returns a tuple(text, start).
+        
+        If there is a selection, start is the position in the text the selection
+        starts, and text is the document text from the (0, 0) to the end of the
+        selection. If there is no selection, the whole document text is returned
+        and start = 0.
+        """
+        docRange = self.doc.documentRange()
+        selRange = self.view.selectionRange()
+        if self.view.selection() and selRange.start().position() != selRange.end().position():
+            text = self.textToCursor(selRange.end())
+            line, column = selRange.start().position()
+            return text, cursorToPosition(line, column, text)
+        return self.text(), 0
+        
     def replaceSelectionWith(self, text, keepSelection=True):
         """
         Convenience method. Replaces the selection (if any) with text.
@@ -811,6 +889,175 @@ class Document(DBusItem):
         return bool(ok and flags & 0x2000000)
 
     
+class UserShortcuts(object):
+    """
+    Manages user-defined keyboard shortcuts.
+    Keyboard shortcuts can be loaded without loading the module they belong to.
+    If a shortcut is triggered, the module is loaded on demand and the action
+    triggered.
+
+    You should subclass this base class and implement the widget() and target()
+    methods.
+    """
+
+    # which config group to store our shortcuts
+    configGroup = "user shortcuts"
+    
+    # the shortcut type to use
+    shortcutContext = Qt.WidgetWithChildrenShortcut
+    
+    def __init__(self, mainwin):
+        self.mainwin = mainwin
+        self._collection = KActionCollection(self.widget())
+        self._collection.setConfigGroup(self.configGroup)
+        self._collection.addAssociatedWidget(self.widget())
+        # load the shortcuts
+        group = KGlobal.config().group(self.configGroup)
+        for key in group.keyList():
+            if group.readEntry(key, QVariant("")).toString():
+                self.addAction(key)
+        self._collection.readSettings()
+    
+    def widget(self):
+        """
+        Should return the widget where the actions should be added to.
+        """
+        pass
+        
+    def target(self):
+        """
+        Should return the object that can further process our actions.
+        It should have the following methods:
+        - actionTriggered(name)
+        - populateAction(action)
+        """
+        pass
+        
+    def addAction(self, name):
+        """
+        (Internal) Create a new action with name name.
+        If existing, return the existing action.
+        """
+        action = self._collection.action(name)
+        if not action:
+            action = self._collection.addAction(name)
+            action.setParent(action.parent()) # Workaround SIP 4.10 BUG
+            action.setShortcutContext(self.shortcutContext)
+            QObject.connect(action, SIGNAL("triggered()"),
+                lambda: self.actionTriggered(name))
+        return action
+    
+    def actionTriggered(self, name):
+        self.target().actionTriggered(name)
+
+    def shortcuts(self):
+        """
+        Returns the list of names we have non-empty shortcuts for.
+        """
+        return [action.objectName()
+            for action in self._collection.actions()
+            if not action.shortcut().isEmpty()]
+                
+    def shortcut(self, name):
+        """
+        Returns the shortcut for action, if existing.
+        """
+        action = self._collection.action(name)
+        if action:
+            if not action.shortcut().isEmpty():
+                return action.shortcut()
+            self.removeShortcut(name)
+    
+    def setShortcut(self, name, shortcut):
+        """
+        Sets the shortcut for the named action.
+        Creates an action if not existing.
+        Deletes the action if set to an empty key sequence.
+        """
+        if not shortcut.isEmpty():
+            action = self.addAction(name)
+            action.setShortcut(shortcut)
+            self._collection.writeSettings(None, True, action)
+        else:
+            self.removeShortcut(name)
+    
+    def removeShortcut(self, name):
+        """
+        Deletes the given action if existing.
+        """
+        action = self._collection.action(name)
+        if action:
+            sip.delete(action)
+            KGlobal.config().group(self.configGroup).deleteEntry(name)
+            
+    def actionCollection(self):
+        """
+        Returns the action collection, fully populated with texts and
+        icons.
+        """
+        for action in self._collection.actions()[:]:
+            if action.shortcut().isEmpty():
+                self.removeShortcut(action.objectName())
+            else:
+                self.target().populateAction(action)
+        return self._collection
+
+    def shakeHands(self, names):
+        """
+        Deletes all actions not in names, and returns a list of the names
+        we have valid actions for.
+        """
+        result = []
+        for action in self._collection.actions()[:]:
+            if action.objectName() not in names:
+                self.removeShortcut(action.objectName()) 
+            elif not action.shortcut().isEmpty():
+                result.append(action.objectName())
+        return result
+        
+
+class StateManager(object):
+    """
+    Manages state and meta-info for documents, like bookmarks
+    and cursor position, etc.
+    """
+    def __init__(self, app):
+        self.app = app
+        self.metainfos = KConfig("metainfos", KConfig.NoGlobals, "appdata")
+        
+    def groupForUrl(self, url, create=False):
+        if not url.isEmpty():
+            encodedurl = url.prettyUrl()
+            if create or self.metainfos.hasGroup(encodedurl):
+                return self.metainfos.group(encodedurl)
+            
+    def loadState(self, doc):
+        group = self.groupForUrl(doc.url())
+        if group:
+            last = group.readEntry("time", QVariant(0.0)).toDouble()[0]
+            # when it is a local file, only load the state when the
+            # file was not modified later
+            if not doc.localPath() or (
+                    os.path.exists(doc.localPath()) and
+                    os.path.getmtime(doc.localPath()) <= last):
+                doc.readConfig(group)
+            
+    def saveState(self, doc):
+        if doc.view and not doc.url().isEmpty():
+            group = self.groupForUrl(doc.url(), True)
+            group.writeEntry("time", QVariant(time.time()))
+            doc.writeConfig(group)
+            group.sync()
+            
+    def cleanup(self):
+        """ Purge entries that are not used for more than a month. """
+        for g in self.metainfos.groupList():
+            last = self.metainfos.group(g).readEntry("time", QVariant(0.0)).toDouble()[0]
+            if (time.time() - last) / 86400 > 31:
+                self.metainfos.deleteGroup(g)
+        self.metainfos.sync()
+
+
 class CursorTranslator(object):
     """
     This object makes a kind of snapshot of a document and makes it
@@ -885,3 +1132,22 @@ def resolvetabs_indices(column, indices):
             return charcol
         charcol += 1
         
+def cursorToPosition(line, column, text):
+    """
+    Returns the character position in text of a cursor with line and column.
+    Line and column both start with 0.
+    Returns -1 if the position falls outside the text.
+    """
+    pos = 0
+    for i in range(line):
+        pos = text.find('\n', pos) + 1
+        if not pos:
+            return -1
+    new = text.find('\n', pos)
+    if new == -1:
+        new = len(text)
+    pos += column
+    if pos > new:
+        return -1
+    return pos
+
