@@ -1,6 +1,6 @@
 # This file is part of the Frescobaldi project, http://www.frescobaldi.org/
 #
-# Copyright (c) 2008, 2009  Wilbert Berendsen
+# Copyright (c) 2008, 2009, 2010 by Wilbert Berendsen
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -51,8 +51,8 @@ class MainWindow(KParts.MainWindow):
     An editor main window.
     
     Emits the following (Python) signals:
-    currentDocumentChanged(Document)
-    aboutToClose() when the window will be closed.
+    - currentDocumentChanged(Document) if the active document changes or its url.
+    - aboutToClose() when the window will be closed.
     """
     def __init__(self, app):
         KParts.MainWindow.__init__(self)
@@ -133,6 +133,7 @@ class MainWindow(KParts.MainWindow):
 
         self._selectionActions = []
         self.setupActions() # Let subclasses add more actions
+        self.setupTools()   # Initialize the tools before loading ui.rc
         self.setStandardToolBarMenuEnabled(True)
         self.createShellGUI(True) # ui.rc is loaded automagically
         
@@ -142,6 +143,7 @@ class MainWindow(KParts.MainWindow):
         self.setupGeneratedMenus()
         self.setAutoSaveSettings()
         self.loadSettings()
+        self.setAcceptDrops(True)
         self.show()
         app.documentCreated.connect(self.addToRecentFiles)
         app.documentMaterialized.connect(self.addDocument)
@@ -183,24 +185,23 @@ class MainWindow(KParts.MainWindow):
         # tool views submenu
         a = KActionMenu(i18n("&Tool Views"), self)
         self.actionCollection().addAction('options_toolviews', a)
-        menu = a.menu()
+        self.toolViewMenu = a.menu() # Save because of BUG in SIP 4.10 loosing this function
         def populate():
-            menu.clear()
+            self.toolViewMenu.clear()
             for tool in self.tools.itervalues():
-                title = menu.addTitle(tool.icon(), tool.title())
-                count = len(menu.children())
-                if not tool.isDocked():
-                    a = menu.addAction(KIcon("tab-detach"), i18n("Dock"))
-                    QObject.connect(a, SIGNAL("triggered()"), tool.dock)
-                elif not tool.isActive():
-                    a = menu.addAction(i18n("Show"))
-                    QObject.connect(a, SIGNAL("triggered()"), tool.show)
-                tool.addMenuActions(menu)
-                # remove title if the tool did not add any menu items
-                if count == len(menu.children()):
-                    sip.delete(title)
-        QObject.connect(menu, SIGNAL("aboutToShow()"), populate)
+                self.toolViewMenu.addSeparator()
+                self.toolViewMenu.addAction(tool.action())
+                tool.addMenuActions(self.toolViewMenu)
+        QObject.connect(self.toolViewMenu, SIGNAL("aboutToShow()"), populate)
 
+    def setupTools(self):
+        """
+        Implement this to create the Tool instances. This is called before the
+        ui.rc file s loaded, so the user can configure the keyboard shortcuts
+        for the tools.
+        """
+        pass
+    
     def setupGeneratedMenus(self):
         """ This should setup menus that are generated on show. """
         # Set up the documents menu so that it shows all open documents.
@@ -223,6 +224,7 @@ class MainWindow(KParts.MainWindow):
                     a.setChecked(True)
                 docGroup.addAction(a)
                 docMenu.addAction(a)
+        docMenu.setParent(docMenu.parent())
         QObject.connect(docMenu, SIGNAL("aboutToShow()"), populateDocMenu)
         
     def act(self, name, texttype, func,
@@ -233,6 +235,7 @@ class MainWindow(KParts.MainWindow):
         else:
             a = self.actionCollection().addAction(name)
             a.setText(texttype)
+        a.setParent(a.parent()) # otherwise SIP 4.10 deletes us (BUG)
         QObject.connect(a, SIGNAL("triggered()"), func)
         if icon: a.setIcon(KIcon(icon))
         if tooltip: a.setToolTip(tooltip)
@@ -278,7 +281,7 @@ class MainWindow(KParts.MainWindow):
         
     def setCurrentDocument(self, doc):
         if self._currentDoc:
-            self._currentDoc.urlChanged.disconnect(self.addToRecentFiles)
+            self._currentDoc.urlChanged.disconnect(self.slotUrlChanged)
             self._currentDoc.captionChanged.disconnect(self.updateCaption)
             self._currentDoc.statusChanged.disconnect(self.updateStatusBar)
             self._currentDoc.selectionChanged.disconnect(self.updateSelection)
@@ -286,7 +289,7 @@ class MainWindow(KParts.MainWindow):
         self._currentDoc = doc
         self.guiFactory().addClient(doc.view)
         self.viewStack.setCurrentWidget(doc.view)
-        doc.urlChanged.connect(self.addToRecentFiles)
+        doc.urlChanged.connect(self.slotUrlChanged)
         doc.captionChanged.connect(self.updateCaption)
         doc.statusChanged.connect(self.updateStatusBar)
         doc.selectionChanged.connect(self.updateSelection)
@@ -310,15 +313,19 @@ class MainWindow(KParts.MainWindow):
     
     def currentDocument(self):
         return self._currentDoc
-            
+    
+    def slotUrlChanged(self, doc=None):
+        self.addToRecentFiles(doc)
+        self.currentDocumentChanged(doc or self._currentDoc)
+
     def updateCaption(self):
         doc = self.currentDocument()
         name = self.showPath.isChecked() and doc.prettyUrl() or doc.documentName()
         if len(name) > 72:
             name = '...' + name[-69:]
         if doc.isModified():
-            self.setCaption(name + " [%s]" % i18n("modified"))
-            self.sb_modified.setPixmap(KIcon("document-properties").pixmap(16))
+            self.setCaption("%s [%s]" % (name, i18n("modified")))
+            self.sb_modified.setPixmap(KIcon("document-save").pixmap(16))
         else:
             self.setCaption(name)
             self.sb_modified.setPixmap(QPixmap())
@@ -346,12 +353,16 @@ class MainWindow(KParts.MainWindow):
         """ Opens a window to edit the keyboard shortcuts """
         dlg = KShortcutsDialog(KShortcutsEditor.AllActions,
             KShortcutsEditor.LetterShortcutsDisallowed, self)
+        self.addActionCollectionsToShortcutsDialog(dlg)
+        dlg.configure()
+    
+    def addActionCollectionsToShortcutsDialog(self, dlg):
+        """ Add KActionCollections to the KShortcutsDialog. """
         dlg.addCollection(self.actionCollection(),
             KGlobal.mainComponent().aboutData().programName())
         if self.view():
             dlg.addCollection(self.view().actionCollection())
-        dlg.configure()
-    
+        
     def editToolbars(self):
         """ Opens a window to edit the toolbar(s) """
         conf = config("MainWindow")
@@ -374,7 +385,8 @@ class MainWindow(KParts.MainWindow):
             self.currentDocument().url().url() or self.app.defaultDirectory(),
             '\n'.join(self.app.fileTypes + ["*|%s" % i18n("All Files")]),
             self, i18n("Open File"))
-        docs = [self.app.openUrl(url, res.encoding) for url in res.URLs
+        encoding = str(res.encoding) # copy it, avoid crash on materialize
+        docs = [self.app.openUrl(url, encoding) for url in res.URLs
                     if not url.isEmpty()]
         if docs:
             docs[-1].setActive()
@@ -394,11 +406,23 @@ class MainWindow(KParts.MainWindow):
 
     def queryClose(self):
         """ Quit the application, also called by closing the window """
-        for d in self.app.documents[:]: # iterate over a copy
+        # First, close the modified documents, if any. (This way, if the user
+        # cancels the first close dialog, all documents are still there.)
+        unmodified = []
+        for d in self.app.history[::-1]: # iterate over a copy, current first
             if d.isModified():
                 d.setActive()
-            if not d.close(True):
-                return False
+                if not d.close(True):
+                    return False # cancelled
+            else:
+                unmodified.append(d)
+        # Then close the unmodified documents. We keep this list ourselves,
+        # because MainApp always creates a new document if the last one is
+        # closed. This way we at least prevent app from creating a new document
+        # twice.
+        self.currentDocumentChanged.clear() # disconnect all tools etc.
+        for d in unmodified:
+            d.close(False)
         # save some settings
         self.saveSettings()
         self.aboutToClose()
@@ -431,6 +455,16 @@ class MainWindow(KParts.MainWindow):
     def createViewTabBar(self):
         return ViewTabBar(self)
 
+    def dragEnterEvent(self, event):
+        event.setAccepted(KUrl.List.canDecode(event.mimeData()))
+        
+    def dropEvent(self, event):
+        if KUrl.List.canDecode(event.mimeData()):
+            urls = KUrl.List.fromMimeData(event.mimeData())
+            docs = map(self.app.openUrl, urls)
+            if docs:
+                docs[-1].setActive()
+
 
 class ViewTabBar(QTabBar):
     """
@@ -494,17 +528,14 @@ class ViewTabBar(QTabBar):
         tab = self.tabAt(ev.pos())
         if tab == -1:
             return
-        menu = self.contextMenu(self.docs[tab])
-        if menu:
-            menu.popup(ev.globalPos())
-    
-    def contextMenu(self, doc, menu = None):
+        menu = KMenu()
+        self.addMenuActions(menu, self.docs[tab])
+        menu.exec_(ev.globalPos())
+
+    def addMenuActions(self, menu, doc):
         """
-        Return a menu with actions relevant for the document.
-        If the menu is given, actions are added to the given menu.
+        Populate the menu with actions relevant for the document.
         """
-        if menu is None:
-            menu = KMenu(self.mainwin)
         g = KStandardGuiItem.save()
         a = menu.addAction(g.icon(), g.text())
         QObject.connect(a, SIGNAL("triggered()"), doc.save)
@@ -515,7 +546,6 @@ class ViewTabBar(QTabBar):
         g = KStandardGuiItem.close()
         a = menu.addAction(g.icon(), g.text())
         QObject.connect(a, SIGNAL("triggered()"), doc.close)
-        return menu
         
 
 class TabBar(KMultiTabBar):
@@ -536,13 +566,14 @@ class TabBar(KMultiTabBar):
         self._tools.append(tool)
         self.appendTab(tool.icon().pixmap(16), tool._id, tool.title())
         tab = self.tab(tool._id)
+        tab.setParent(tab.parent()) # because of SIP 4.10 BUG loosing outer scope
         tab.setFocusPolicy(Qt.NoFocus)
         tab.setToolTip("<b>%s</b><br/>%s" % (tool.title(),
             i18n("Right-click for tab options")))
         tab.setContextMenuPolicy(Qt.CustomContextMenu)
         QObject.connect(tab, SIGNAL("clicked()"), tool.toggle)
         QObject.connect(tab, SIGNAL("customContextMenuRequested(const QPoint&)"),
-            lambda pos: tool.contextMenu().popup(tab.mapToGlobal(pos)))
+            lambda pos: tool.showContextMenu(tab.mapToGlobal(pos)))
 
     def removeTool(self, tool):
         self._tools.remove(tool)
@@ -677,7 +708,7 @@ class Tool(object):
     __instance_counter = 0
     
     def __init__(self, mainwin, name,
-            title="", icon="", dock=Right,
+            title="", icon="", key="", dock=Right,
             widget=None):
         self._id = Tool.__instance_counter
         self._active = False
@@ -689,6 +720,11 @@ class Tool(object):
         self.mainwin = mainwin
         self.name = name
         mainwin.tools[name] = self
+        action = KAction(mainwin) # action to toggle our view
+        QObject.connect(action, SIGNAL("triggered()"), self.slotAction)
+        mainwin.actionCollection().addAction("tool_%s" % name, action)
+        if key:
+            action.setShortcut(KShortcut(key))
         self.widget = widget
         self.setTitle(title)
         self.setIcon(icon)
@@ -696,6 +732,18 @@ class Tool(object):
         Tool.__instance_counter += 1
         self.loadSettings()
     
+    def action(self):
+        return self.mainwin.actionCollection().action("tool_%s" % self.name)
+    
+    def slotAction(self):
+        """
+        Called when our action is triggered.
+        Default behaviour is to toggle the visibility of our tool.
+        Override this to implement other behaviour when our action is called
+        (e.g. focus instead of hide).
+        """
+        self.toggle()
+        
     def materialize(self):
         if self.widget is None:
             self.widget = self.factory()
@@ -716,6 +764,7 @@ class Tool(object):
             sip.delete(self.widget)
         if self._dialog:
             sip.delete(self._dialog)
+        sip.delete(self.action())
         del self._dock, self.widget, self._dialog
         del self.mainwin.tools[self.name]
 
@@ -791,6 +840,7 @@ class Tool(object):
         
     def setIcon(self, icon):
         self._icon = icon and KIcon(icon) or KIcon()
+        self.action().setIcon(self._icon)
         self.updateState()
 
     def title(self):
@@ -798,6 +848,7 @@ class Tool(object):
     
     def setTitle(self, title):
         self._title = title
+        self.action().setText(self._title)
         self.updateState()
             
     def updateState(self):
@@ -807,10 +858,9 @@ class Tool(object):
         else:
             self._dialog.updateState()
             
-    def contextMenu(self):
+    def showContextMenu(self, globalPos):
         """
-        Return a popup menu to manipulate this tool.
-        Do not subclass this method, but use addMenuActions instead.
+        Show a popup menu to manipulate this tool.
         """
         m = KMenu(self.mainwin)
         places = [place for place in Left, Right, Top, Bottom
@@ -831,7 +881,8 @@ class Tool(object):
             m.addSeparator()
             a = m.addAction(KIcon("help-contextual"), KStandardGuiItem.help().text())
             QObject.connect(a, SIGNAL("triggered()"), self.help)
-        return m
+        QObject.connect(m, SIGNAL("aboutToHide()"), m.deleteLater)
+        m.popup(globalPos)
 
     def addMenuActions(self, menu):
         """
@@ -875,10 +926,10 @@ class KPartTool(Tool):
     # set this to the name of the app containing this part
     _partappname = ""
     
-    def __init__(self, mainwin, name, title="", icon="", dock=Right):
+    def __init__(self, mainwin, name, title="", icon="", key="", dock=Right):
         self.part = None
         self.failed = False
-        Tool.__init__(self, mainwin, name, title, icon, dock)
+        Tool.__init__(self, mainwin, name, title, icon, key, dock)
     
     def factory(self):
         if self.part:
@@ -888,7 +939,7 @@ class KPartTool(Tool):
             part = factory.create(self.mainwin)
             if part:
                 self.part = part
-                QObject.connect(part, SIGNAL("destroyed()"), self.slotDestroyed)
+                QObject.connect(part, SIGNAL("destroyed()"), self.slotDestroyed, Qt.DirectConnection)
                 QTimer.singleShot(0, self.partLoaded)
                 return part.widget()
         self.failed = True
