@@ -1,6 +1,6 @@
 # This file is part of LilyDOM, http://lilykde.googlecode.com/
 #
-# Copyright (c) 2008, 2009  Wilbert Berendsen
+# Copyright (c) 2008, 2009, 2010 by Wilbert Berendsen
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -172,6 +172,17 @@ class Node(object):
         """ Remove all children """
         del self[:]
 
+    def copy(self):
+        """ Return a deep copy of the node and its children """
+        obj = self.__class__.__new__(self.__class__)
+        for name, value in vars(self).items():
+            name.startswith("_") or setattr(obj, name, value)
+        obj._parent = None
+        obj._children = []
+        for n in self:
+            obj.append(n.copy())
+        return obj
+            
     def ancestors(self):
         """ climb the tree up over the parents """
         node = self.parent()
@@ -505,6 +516,14 @@ class Container(LyNode):
             return "".join(res)
 
 
+class Block(Container):
+    """ 
+    A vertical container type that puts everything on a new line.
+    """
+    defaultSpace = "\n"
+    before, after = 1, 1
+
+
 class Document(Container):
     """ 
     A container type that puts everything on a new line.
@@ -565,9 +584,9 @@ class BlockComment(Comment):
     def ly(self, printer):
         text = self.text.replace('%}', '')
         if '\n' in text:
-            return "%{\n%s\n%}" % text
+            return "%%{\n%s\n%%}" % text
         else:
-            return "%{ %s %}" % text
+            return "%%{ %s %%}" % text
             
 
 class QuotedString(Text):
