@@ -1,6 +1,6 @@
 # This file is part of the Frescobaldi project, http://www.frescobaldi.org/
 #
-# Copyright (c) 2008, 2009  Wilbert Berendsen
+# Copyright (c) 2008, 2009, 2010 by Wilbert Berendsen
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -49,8 +49,18 @@ keywords = (
     'midi',
     'layout',
     'with',
+    'context',
 )
 
+keywords_completion = (
+    'score {}',
+    'header {}',
+    'paper {}',
+    'midi {}',
+    'layout {}',
+    'with {}',
+    'context {}',
+)
 
 musiccommands = (
     'accent',
@@ -434,7 +444,23 @@ musiccommands = (
     'voiceTwoStyle',
     'whiteTriangleMarkup',
     'withMusicProperty',
-    )
+)
+
+
+musiccommands_completion = (
+    'addlyrics {}',
+    'alternative {}',
+    'chordmode {}',
+    'drummode {}',
+    'figuremode {}',
+    'keepWithTag #\'',
+    'lyricmode {}',
+    'notemode {}',
+    'relative c\' {}',
+    'removeWithTag #\'',
+    'tag #\'',
+    'tweak #\'',
+)
 
 
 modes = (
@@ -450,127 +476,141 @@ modes = (
 )
 
 
-markupcommands = (
-    'arrow-head',
-    'backslashed-digit',
-    'beam',
-    'char',
+markupcommands_nargs = (
+# no arguments
+(
     'doubleflat',
     'doublesharp',
-    'draw-circle',
-    'draw-line',
-    'epsfile',
-    'filled-box',
     'flat',
-    'fret-diagram',
-    'fret-diagram-terse',
-    'fret-diagram-verbose',
-    'fromproperty',
-    'harp-pedal',
-    'justify-field',
-    'justify-string',
-    'lookup',
-    'markalphabet',
-    'markletter',
-    'musicglyph',
     'natural',
-    'note',
-    'note-by-number',
     'null',
     'semiflat',
     'semisharp',
     'sesquiflat',
     'sesquisharp',
     'sharp',
-    'simple',
-    'slashed-digit',
-    'stencil',
     'strut',
-    'tied-lyric',
-    'triangle',
-    'verbatim-file',
-    'wordwrap-field',
-    'wordwrap-string',
-    # with args
-    'abs-fontsize',
+),
+# one argument
+(
+    'backslashed-digit',
     'bold',
     'box',
     'bracket',
     'caps',
     'center-align',
     'center-column',
+    'char',
     'circle',
     'column',
-    'combine',
     'concat',
     'dir-column',
+    'draw-line',
     'dynamic',
     'fill-line',
     'finger',
     'fontCaps',
-    'fontsize',
-    'fraction',
-    'general-align',
-    'halign',
+    'fret-diagram',
+    'fret-diagram-terse',
+    'fret-diagram-verbose',
+    'fromproperty',
+    'harp-pedal',
     'hbracket',
-    'hcenter',
-    'hcenter-in',
     'hspace',
     'huge',
     'italic',
     'justify',
+    'justify-field',
+    'justify-string',
     'large',
     'larger',
     'left-align',
     'left-column',
     'line',
-    'lower',
-    'magnify',
-    'markup',
-    'markuplines',
+    'lookup',
+    'markalphabet',
+    'markletter',
     'medium',
+    'musicglyph',
     'normalsize',
     'normal-size-sub',
     'normal-size-super',
     'normal-text',
     'number',
-    'on-the-fly',
-    'override',
-    'pad-around',
-    'pad-markup',
-    'pad-to-box',
-    'pad-x',
-    'page-ref',
     'postscript',
-    'put-adjacent',
-    'raise',
     'right-align',
     'right-column',
     'roman',
-    'rotate',
     'rounded-box',
     'sans',
+    'score',
+    'simple',
+    'slashed-digit',
     'small',
     'smallCaps',
     'smaller',
+    'stencil',
     'sub',
     'super',
     'teeny',
     'text',
+    'tied-lyric',
     'tiny',
-    'translate',
-    'translate-scaled',
     'transparent',
+    'triangle',
     'typewriter',
     'underline',
     'upright',
     'vcenter',
+    'verbatim-file',
     'whiteout',
-    'with-color',
-    'with-dimensions',
-    'with-url',
     'wordwrap',
-    )
+    'wordwrap-field',
+    'wordwrap-string',
+),
+# two arguments
+(
+    'abs-fontsize',
+    'combine',
+    'fontsize',
+    'fraction',
+    'halign',
+    'hcenter-in',
+    'lower',
+    'magnify',
+    'note',
+    'on-the-fly',
+    'override',
+    'pad-around',
+    'pad-markup',
+    'pad-x',
+    'raise',
+    'rotate',
+    'translate',
+    'translate-scaled',
+    'with-color',
+    'with-url',
+),
+# three arguments
+(
+    'arrow-head',
+    'beam',
+    'draw-circle',
+    'eps-file',
+    'filled-box',
+    'general-align',
+    'note-by-number',
+    'pad-to-box',
+    'page-ref',
+    'with-dimensions',
+),
+# four arguments
+(
+    'put-adjacent',
+))
+
+
+markupcommands = sum(markupcommands_nargs, ())
 
 
 markuplistcommands = (
@@ -580,7 +620,7 @@ markuplistcommands = (
     'wordwrap-internal',
     'wordwrap-lines',
     'wordwrap-string-internal',
-    )
+)
 
 
 contexts = (
@@ -610,7 +650,7 @@ contexts = (
     'VaticanaStaff',
     'VaticanaVoice',
     'Voice',
-    )
+)
 
 
 engravers = (
@@ -744,10 +784,11 @@ engravers = (
     'Vertical_align_engraver',
     'Vertically_spaced_contexts_engraver',
     'Volta_engraver',
-    )
+)
 
 
 midi_instruments = (
+    # (1-8 piano)
     'acoustic grand',
     'bright acoustic',
     'electric grand',
@@ -917,7 +958,7 @@ midi_instruments = (
     'mt-32 drums',
     'cm-64 kit',
     'cm-64 drums',
-    )
+)
 
 
 grobs = (
@@ -1049,7 +1090,7 @@ grobs = (
     'VoiceFollower',
     'VoltaBracket',
     'VoltaBracketSpanner',
-    )
+)
 
 
 def schemeprops(grob = None):
@@ -1281,11 +1322,11 @@ all_user_grob_properties = (
     'Y-offset',
     'zigzag-length',
     'zigzag-width',
-    )
+)
 
 
 schemefuncs = (
-    'set-accidental-style',
+    'set-accidental-style \'',
     'set-global-staff-size',
     'set-octavation',
     'define-public',
@@ -1310,8 +1351,11 @@ schemefuncs = (
     'make-center-markup',
     'make-column-markup',
     'make-musicglyph-markup',
-    )
-    
+    'color?',
+    'rgb-color',
+    'x11-color \'',
+)
+
 
 headervars = (
     'dedication',
@@ -1344,7 +1388,7 @@ headervars = (
     'lastupdated',
     'texidoc',
     'footer',
-    )
+)
     
 
 papervars = (
@@ -1397,14 +1441,14 @@ papervars = (
     'scoreTitleMarkup',
     'tocItemMarkup',
     'tocTitleMarkup',
-    )
+)
 
 
 layoutvars = (
     'indent',
     'short-indent',
     'system-count',
-    )
+)
 
 
 contextproperties = (
@@ -1559,7 +1603,7 @@ contextproperties = (
     'voltaOnThisStaff', 
     'voltaSpannerDuration', 
     'whichBar', 
-    )
+)
 
 
 repeat_types = (
@@ -1567,7 +1611,7 @@ repeat_types = (
     'percent',
     'volta',
     'tremolo',
-    )
+)
 
 
 accidentalstyles = (
@@ -1585,7 +1629,7 @@ accidentalstyles = (
     'teaching',
     'no-reset',
     'forget',
-    )
+)
 
 
 clefs_plain = (
@@ -1594,6 +1638,7 @@ clefs_plain = (
     'G',
     'alto',
     'C',
+    'tenor',
     'bass',
     'subbass',
     'F',
@@ -1604,13 +1649,24 @@ clefs_plain = (
     'baritone',
     'percussion',
     'tab',
-    )
+)
     
 
 clefs = clefs_plain + (
     'treble_8',
     'bass_8',
-    )
+)
+
+
+break_visibility = (
+    'all-invisible',
+    'begin-of-line-visible',
+    'end-of-line-visible',
+    'all-visible',
+    'begin-of-line-invisible',
+    'end-of-line-invisible',
+    'center-invisible',
+)
 
 
 set_context_re = re.compile(r'\\(un)?set\s+(' + '|'.join(contexts) + r')\s*.\s*$')
